@@ -400,65 +400,78 @@ int m, n;
 }
 
 /* Move supply from one unit to another.  Don't move more than is possible;
-   check both from and to amounts and capacities. */
+   check both from and to amounts and capacities.  This routine will also
+   transfer to the side's treasury if necessary to handle overflow. */
 
 int
 transfer_supply(from, to, m, amount)
 Unit *from, *to;
 int m, amount;
 {
-    int origfrom = from->supply[m], origto = to->supply[m];
+    int amt2, origfrom = from->supply[m], origto = to->supply[m];
+    Side *side = NULL;
 
+    /* Constrain the amount of supply that may be unloaded. */
     amount = min(amount, origfrom);
-    amount = min(amount, um_storage_x(to->type, m) - origto);
     if (um_unload_max(from->type, m) >= 0) {
 	amount = min(amount, um_unload_max(from->type, m));
     }
     if (um_load_max(to->type, m) >= 0) {
 	amount = min(amount, um_load_max(to->type, m));
     }
+    amt2 = min(amount, um_storage_x(to->type, m) - origto);
+    if (um_gives_to_treasury(to->type, m)) {
+	side = (to->side ? to->side : indepside);
+	side->treasury[m] += (amount - amt2);
+    } else {
+	amount = amt2;
+    }
     from->supply[m] -= amount;
-    to->supply[m] += amount;
+    to->supply[m] += amt2;
     /* Make sure any displays of supply levels see the transfer. */
-    update_unit_display(from->side, from, TRUE); 
-    update_unit_display(to->side, to, TRUE); 
-    Dprintf("%s (had %d) transfers %d %s to %s (had %d)\n",
+    update_unit_display(from->side, from, TRUE);
+    update_unit_display(to->side, to, TRUE);
+    if (amount != amt2) {
+	update_side_display(from->side, side, TRUE);
+	update_side_display(to->side, side, TRUE);
+    }
+    Dprintf("%s (had %d) transfers %d %s to %s (had %d) (%d to side)\n",
 	    unit_desig(from), origfrom, amount, m_type_name(m),
-	    unit_desig(to), origto);
+	    unit_desig(to), origto, (amount - amt2));
     return amount;
 }
 
-/* Research action. */
+/* Develop action. */
 
-/* If a side's tech level is under its max, research can increase it. */
+/* If a side's tech level is under its max, develop can increase it. */
 
 int
-prep_research_action(unit, unit2, u3)
+prep_develop_action(unit, unit2, u3)
 Unit *unit, *unit2;
 int u3;
 {
     if (unit == NULL || unit->act == NULL || unit2 == NULL)
       return FALSE;
-    unit->act->nextaction.type = ACTION_RESEARCH;
+    unit->act->nextaction.type = ACTION_DEVELOP;
     unit->act->nextaction.args[0] = u3;
     unit->act->nextaction.actee = unit2->id;
     return TRUE;
 }
 
 int
-net_prep_research_action(unit, unit2, u3)
+net_prep_develop_action(unit, unit2, u3)
 Unit *unit, *unit2;
 int u3;
 {
     int rslt;
 
-    rslt = prep_research_action(unit, unit2, u3);
+    rslt = prep_develop_action(unit, unit2, u3);
     broadcast_next_action(unit);
     return rslt;
 }
 
 int
-do_research_action(unit, unit2, u3)
+do_develop_action(unit, unit2, u3)
 Unit *unit, *unit2;
 int u3;
 {
@@ -466,7 +479,7 @@ int u3;
     Side *side = unit2->side;
 
     oldtech = side->tech[u3];
-    side->tech[u3] += prob_fraction(uu_tech_per_research(u2, u3));
+    side->tech[u3] += prob_fraction(uu_tech_per_develop(u2, u3));
     /* Silently apply the per-side-per-turn limit on tech gains. */
     lim =  side->inittech[u3] + u_tech_per_turn_max(u3);
     if (side->tech[u3] > lim)
@@ -477,12 +490,12 @@ int u3;
     }
     /* Adjust the tech levels of any related unit types. */
     adjust_tech_crossover(side, u3);
-    use_up_acp(unit, uu_acp_to_research(u2, u3));
+    use_up_acp(unit, uu_acp_to_develop(u2, u3));
     return A_ANY_DONE;
 }
 
 int
-check_research_action(unit, unit2, u3)
+check_develop_action(unit, unit2, u3)
 Unit *unit, *unit2;
 int u3;
 {
@@ -497,22 +510,22 @@ int u3;
       return A_ANY_ERROR;
     u = unit->type;  u2 = unit2->type;
     side = unit->side;
-    /* Independent units don't do research. */
+    /* Independent units don't do develop. */
     if (side == NULL)
       return A_ANY_ERROR;
-    /* This unit must be of a type that can research the given type. */
-    if (uu_acp_to_research(u2, u3) < 1)
+    /* This unit must be of a type that can develop the given type. */
+    if (uu_acp_to_develop(u2, u3) < 1)
       return A_ANY_CANNOT_DO;
-    if (!can_have_enough_acp(unit, uu_acp_to_research(u2, u3)))
+    if (!can_have_enough_acp(unit, uu_acp_to_develop(u2, u3)))
       return A_ANY_CANNOT_DO;
     /* Max tech level means there's nothing more to learn. */
     if (side->tech[u3] >= u_tech_max(u3))
       return A_ANY_ERROR;
-    if (!has_enough_acp(unit, uu_acp_to_research(u2, u3)))
+    if (!has_enough_acp(unit, uu_acp_to_develop(u2, u3)))
       return A_ANY_NO_ACP;
     /* Check that the unit has any required supplies. */
     for_all_material_types(m) {
-	if (unit2->supply[m] < um_to_research(u2, m))
+	if (unit2->supply[m] < um_to_develop(u2, m))
 	  return A_ANY_NO_MATERIAL;
     }
     return A_ANY_OK;
@@ -738,7 +751,7 @@ do_create_in_action(unit, unit2, u3, dest)
 Unit *unit, *unit2, *dest;
 int u3;
 {
-    int u2 = unit2->type, m;
+    int u2 = unit2->type, m, consum, consum2;
     Unit *newunit;
 
     /* Doublecheck that the new unit is still allowed. */
@@ -763,7 +776,14 @@ int u3;
 	    count_gain(unit2->side, newunit->type, build_gain);
 	    /* Consume the creator's supplies as specified. */
 	    for_all_material_types(m) {
-		unit2->supply[m] -= um_consumption_on_creation(u3, m);
+		consum = um_consumption_on_creation(u3, m);
+		consum2 = 0;
+		if (consum > unit2->supply[m]) {
+		    consum2 = consum - unit2->supply[m];
+		    consum = unit2->supply[m];
+		}
+		unit2->supply[m] -= consum;
+		unit2->side->treasury[m] -= consum2;
 	    }
 	}
 	use_up_acp(unit, uu_acp_to_create(u2, u3));
@@ -798,7 +818,7 @@ check_create_common(unit, unit2, u3, x, y)
 Unit *unit, *unit2;
 int u3;
 {
-    int u, u2, m, tp;
+    int u, u2, m, tp, totsup;
 
     if (!in_play(unit))
       return A_ANY_ERROR;
@@ -809,19 +829,12 @@ int u3;
     u = unit->type;  u2 = unit2->type;
     if (uu_acp_to_create(u2, u3) < 1)
       return A_ANY_CANNOT_DO;
-      
-// --- IDLE MPLAYER BUG FIX -----------------------------------------      
-      
-/* Don't permit the creation of units that are not allowed on this side. 
-It causes the mplayer to be stuck building a unit it can never complete. */
-
-   if (!type_allowed_on_side(u3, unit->side))
-      return A_ANY_CANNOT_DO;
-   
-// -------------------------------------------------------------   
-         
     if (!can_have_enough_acp(unit, uu_acp_to_create(u2, u3)))
       return A_ANY_CANNOT_DO;
+    /* Don't permit the creation of units that are not allowed on this
+       side. */
+    if (!type_allowed_on_side(u3, unit->side))
+      return A_ANY_CANNOT_DO;
     /* Check the tech level of the side. */
     if (u_tech_to_build(u3) > 0) {
 	if (unit->side == NULL)
@@ -841,9 +854,10 @@ It causes the mplayer to be stuck building a unit it can never complete. */
         && !uu_occ_can_build(unit2->transport->type, u2))
       return A_ANY_ERROR;
     for_all_material_types(m) {
-    	if (unit2->supply[m] < um_to_create(u3, m))
+	totsup = unit2->supply[m] + unit2->side->treasury[m];
+    	if (totsup < um_to_create(u3, m))
     	  return A_ANY_NO_MATERIAL;
-    	if (unit2->supply[m] < um_consumption_on_creation(u3, m))
+    	if (totsup < um_consumption_on_creation(u3, m))
     	  return A_ANY_NO_MATERIAL;
     }
     if (!has_enough_acp(unit, uu_acp_to_create(u2, u3)))
@@ -925,7 +939,7 @@ do_create_at_action(unit, unit2, u3, x, y, z)
 Unit *unit, *unit2;
 int u3, x, y, z;
 {
-    int u2 = unit2->type, m;
+    int u2 = unit2->type, m, consum, consum2;
     Unit *newunit;
 
     /* Doublecheck that the new unit is still allowed. */
@@ -946,21 +960,9 @@ int u3, x, y, z;
 	    enter_cell(newunit, x, y);
 	    enter_transport(unit2, newunit);
 	} else {
-	    /* This should never happen. */
- 
- //--- FULL CITY BUG FIX ---------------------------------------
-
-	/* It does. It just means that both the city itself and all adjacent cells are full. 
-	This should not crash the game. Just disband the new unit instead, and notify. */
- 
-	    notify(unit->side, "%s is full. New unit disbanded on creation.", unit->name);			
-	    kill_unit(newunit, H_UNIT_DISBANDED);
-	    return A_ANY_ERROR;	    
-
-//	    run_error("construction/occupation complications");
-
-// --------------------------------------------------------
-
+	    /* This should never happen.  If it does, then the preflighting
+	       in check_create_at_action has made a mistake. */
+	    run_error("construction/occupation complications");
 	}
 	/* (should) set its altitude? */
 	/* Unit might be complete right away. */
@@ -975,7 +977,14 @@ int u3, x, y, z;
 	    count_gain(unit2->side, newunit->type, build_gain);
 	    /* Consume the creator's supplies as specified. */
 	    for_all_material_types(m) {
-		unit2->supply[m] -= um_consumption_on_creation(u3, m);
+		consum = um_consumption_on_creation(u3, m);
+		consum2 = 0;
+		if (consum > unit2->supply[m]) {
+		    consum2 = consum - unit2->supply[m];
+		    consum = unit2->supply[m];
+		}
+		unit2->supply[m] -= consum;
+		unit2->side->treasury[m] -= consum2;
 	    }
 	}
 	use_up_acp(unit, uu_acp_to_create(u2, u3));
@@ -998,6 +1007,8 @@ int u3, x, y, z;
     rslt = check_create_common(unit, unit2, u3, x, y);
     if (rslt != A_ANY_OK)
       return rslt;
+    /* Check that the desired cell has room for the unit, either by
+       stacking, or by having the builder occupy its construction work. */
     if (!(type_can_occupy_cell(u3, x, y)
           || (can_occupy_type(unit2, u3)
               && type_can_occupy_cell_without(u3, x, y, unit2))))
@@ -1211,21 +1222,8 @@ Unit *unit;
     make_up_unit_name(unit);
     /* It also starts viewing its surroundings. */
     cover_area(unit->side, unit, unit->transport, -1, -1, unit->x, unit->y);
-
-// --- USER LAYER FIX -----------------------------------------
-
+    /* Kick out enemy users if we control this cell. */
     kick_out_enemy_users(unit->side, unit->x, unit->y);
-
-//    /* Kick out enemy users if we control this cell. */
-//    if (user_at(unit->x, unit->y) != NULL 
-//    	&& (!user_at(unit->x, unit->y)->side 
-//    	|| !trusted_side(user_at(unit->x, unit->y)->side, unit->side))) {
-//		user_at(unit->x, unit->y)->usedcells -=1;
-//    		set_user_at(unit->x, unit->y, NULL);
-// 	}
-         
-// --------------------------------------------------------
-
     /* Set all the supplies up to their unit-just-completed levels. */
     for_all_material_types(m) {
 	unit->supply[m] = max(unit->supply[m], um_completed_supply(u, m));
@@ -1330,7 +1328,7 @@ Unit *unit, *unit2, *unit3;
     if (unit2->hp < uu_hp_to_repair(u2, u3))
       return A_ANY_ERROR;
     for_all_material_types(m) {
-	if (unit2->supply[m] < um_to_repair(u2, m))
+	if (unit2->supply[m] < um_to_repair(u3, m))
 	  return A_ANY_NO_MATERIAL;
 	if (unit2->supply[m] < um_consumption_per_repair(u3, m))
 	  return A_ANY_NO_MATERIAL;
@@ -1928,13 +1926,8 @@ int x, y, dir, t;
     /* Let everybody see what has happened. */
     for_all_sides(side) {
 	if (active_display(side)) {
-	    if (terrain_visible(side, x, y)) {
-		update_cell_display(side, x, y, TRUE);
-	    }
-	    if (t_subtype(t) != coatingsubtype) {
-		if (point_in_dir(x, y, dir, &x1, &y1) && terrain_visible(side, x1, y1))
-		  update_cell_display(side, x1, y1, TRUE);
-    	    }
+	    if (terrain_visible(side, x, y))
+	      update_cell_display(side, x, y, UPDATE_ALWAYS | UPDATE_ADJ);
     	}
     }
     use_up_acp(unit, (newval != oldval ? ut_acp_to_add_terrain(u, t) : 1));
@@ -2057,13 +2050,8 @@ int x, y, dir, t;
     /* Let everybody see what has happened. */
     for_all_sides(side) {
 	if (active_display(side)) {
-	    if (terrain_visible(side, x, y)) {
-		update_cell_display(side, x, y, TRUE);
-	    }
-	    if (t_subtype(t) != coatingsubtype) {
-		if (point_in_dir(x, y, dir, &x1, &y1) && terrain_visible(side, x1, y1))
-		  update_cell_display(side, x1, y1, TRUE);
-    	    }
+	    if (terrain_visible(side, x, y))
+	      update_cell_display(side, x, y, UPDATE_ALWAYS | UPDATE_ADJ);
     	}
     }
     use_up_acp(unit, (newval != oldval ? ut_acp_to_remove_terrain(u, t) : 1));
@@ -2233,9 +2221,10 @@ Action *action;
 	}
 	/* Show other sides that some action has occurred. */
 	update_side_display_all_sides(unit->side, TRUE);
-	/* If of a type that might be spotted if it does anything, check
-	   each side to see if they notice.  Note that the type is from
-	   *before* the action, not after (some actions may cause type changes). */
+	/* If of a type that might be spotted if it does anything,
+	   check each side to see if they notice.  Note that the type
+	   is from *before* the action, not after (some actions may
+	   cause type changes). */
 	if (u_spot_action(u)
 	    && !all_see_all
 	    && !u_see_always(u)
@@ -2246,8 +2235,9 @@ Action *action;
 		}
 	    }
 	}
-	/* Check on any scorekeepers that run after each action.  Note that one or more
-	   sides may be out of the game after this returns. */
+	/* Check on any scorekeepers that run after each action.  Note
+	   that one or more sides may be out of the game after this
+	   returns. */
 	if (any_post_action_scores)
 	  check_post_action_scores();
     } else {
@@ -2524,20 +2514,20 @@ int acp;
 /* Functions returning general abilities of a unit. */
 
 int
-can_research(unit)
+can_develop(unit)
 Unit *unit;
 {
-    return type_can_research(unit->type);
+    return type_can_develop(unit->type);
 }
 
 int
-type_can_research(u)
+type_can_develop(u)
 int u;
 {
     int u2;
 	
     for_all_unit_types(u2) {
-	if (uu_acp_to_research(u, u2) > 0)
+	if (uu_acp_to_develop(u, u2) > 0)
 	  return TRUE;
     }
     return FALSE;
@@ -2627,6 +2617,71 @@ int u;
     return FALSE;
 }
 
+/* This is true if the given location has some kind of material for the
+   unit to extract. */
+
+int
+can_extract_at(Unit *unit, int x, int y, int *mp)
+{
+    int m;
+    Unit *unit2;
+
+    /* Can't do anything with an unseen location. */
+    if (unit->side != NULL
+	&& terrain_view(unit->side, x, y) == UNSEEN)
+      return FALSE;
+    for_all_material_types(m) {
+	if (um_acp_to_extract(unit->type, m) > 0) {
+	    /* Look for case of extraction from terrain. */
+	    if (any_cell_materials_defined()
+		&& cell_material_defined(m)
+		&& material_at(x, y, m) > 0) {
+		*mp = m;
+		return TRUE;
+	    }
+	    /* Then look for extraction from independent unit. */
+	    for_all_stack(x, y, unit2) {
+		if (in_play(unit2)
+		    && indep(unit2)
+		    && unit2->supply[m] > 0) {
+		    *mp = m;
+		    return TRUE;
+		}
+	    }
+	}
+    }
+    return FALSE;
+}
+
+/* This is true if the given location has some kind of material for the
+   unit to transfer. */
+
+int
+can_load_at(Unit *unit, int x, int y, int *mp)
+{
+    int m;
+    Unit *unit2;
+
+    /* Can't do anything with an unseen location. */
+    if (unit->side != NULL
+	&& terrain_view(unit->side, x, y) == UNSEEN)
+      return FALSE;
+    for_all_material_types(m) {
+	if (um_acp_to_load(unit->type, m) > 0) {
+	    for_all_stack(x, y, unit2) {
+		if (in_play(unit2)
+		    && unit2->side == unit->side
+		    && u_acp(unit2->type) == 0
+		    && um_acp_to_unload(unit2->type, m) > 0) {
+		    *mp = m;
+		    return TRUE;
+		}
+	    }
+	}
+    }
+    return FALSE;
+}
+
 /* This tests whether the given unit is capable of doing repair. */
 
 int
