@@ -1,5 +1,5 @@
 /* Functions common to all AIs.
-   Copyright (C) 1992-1997 Stanley T. Shebs.
+   Copyright (C) 1992-1997, 1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -290,33 +290,26 @@ char *ainame;
 {
     Side *side1;
 
-    /* If there is a human player and the decision has not been
-       delegated, allow the human to make the final decision. */
-    if (side_has_display(side) && !side->ai_may_resign) {
-	notify(side, "Your AI %s recommends resignation", ainame);
+    /* Can't resign if we've been forbidden to do so. */
+    if (!side->ai_may_resign) {
+	/* If there is a human player, speak up. */
+	if (side_has_display(side))
+	  notify(side, "Your AI %s recommends resignation", ainame);
 	return;
     }
-
-// --- SUPPORT FOR TURNING OFF AUTOMATIC AI RESIGNATION --------------------
-
-    /* Don't proceed if autoresignation has been disabled. */
-    else if (!side->ai_may_resign)
-	return;
-
-// ----------------------------------------------------------------
-
-    /* (should have to give units to controlling side if there is one) */
-    /* (should controlling side be able to disallow resignation?) */
     /* Try to give away all of our units to an ally. */
     for_all_sides(side1) {
 	if (side != side1 && allied_side(side, side1) && side1->ingame) {
+	    if (side_has_display(side))
+	      notify(side, "Your AI resigns, giving everthing to %s!",
+		     short_side_title(side1));
 	    net_resign_game(side, side1);
 	    return;
 	}
     }
-    /* (should give to any positively-regarded side?) */
-    /* No allies left in game, let everything become independent. */
-    /* (should disband all units that can be disbanded?) */
+    /* No allies left in game, leave everything to its fate. */
+    if (side_has_display(side))
+      notify(side, "Your AI resigns!");
     net_resign_game(side, NULL);
 }
 
