@@ -315,7 +315,6 @@ int x, y, z, n;
     if (!has_enough_acp(unit, 1))
       return A_ANY_NO_ACP;
     /* Check whether we can attack from inside a transport. */
-    /* (although this might be legit if unit is assumed to leave transport first) */
     if (unit2->transport && uu_occ_combat(u2, unit2->transport->type) == 0)
       return A_ANY_ERROR;
     u2x = unit2->x;  u2y = unit2->y;  u2z = unit2->z;
@@ -385,10 +384,10 @@ Unit *atker, *defender;
     see_exact(as, ox, oy);
     see_exact(os, ax, ay);
     see_exact(os, ox, oy);
-    update_cell_display(as, ax, ay, TRUE);
-    update_cell_display(as, ox, oy, TRUE);
-    update_cell_display(os, ax, ay, TRUE);
-    update_cell_display(os, ox, oy, TRUE);
+    update_cell_display(as, ax, ay, UPDATE_ALWAYS);
+    update_cell_display(as, ox, oy, UPDATE_ALWAYS);
+    update_cell_display(os, ax, ay, UPDATE_ALWAYS);
+    update_cell_display(os, ox, oy, UPDATE_ALWAYS);
     all_see_cell(ax, ay);
     all_see_cell(ox, oy);
     attempt_to_capture_unit(atker, defender);
@@ -459,10 +458,10 @@ int m;
       update_unit_display(unit->side, unit, TRUE);
     update_unit_display(defender->side, defender, TRUE);
     /* The attacking side also sees the remote cell. */
-    update_cell_display(unit2->side, ox, oy, TRUE);
-    update_cell_display(defender->side, ox, oy, TRUE);
+    update_cell_display(unit2->side, ox, oy, UPDATE_ALWAYS);
+    update_cell_display(defender->side, ox, oy, UPDATE_ALWAYS);
     /* Victim might see something in attacker's cell. */
-    update_cell_display(defender->side, ux, uy, TRUE);
+    update_cell_display(defender->side, ux, uy, UPDATE_ALWAYS);
     /* Actually, everybody might be seeing the combat. */
     all_see_cell(ux, uy);
     all_see_cell(ox, oy);
@@ -595,10 +594,10 @@ int x, y, z, m;
 	/* Each side sees what happened to its unit that is being hit. */
 	update_unit_display(other->side, other, TRUE);
 	/* The attacking side also sees the remote cell. */
-	update_cell_display(unit->side, ox, oy, TRUE);
-	update_cell_display(other->side, ox, oy, TRUE);
+	update_cell_display(unit->side, ox, oy, UPDATE_ALWAYS);
+	update_cell_display(other->side, ox, oy, UPDATE_ALWAYS);
 	/* Victim might see something in attacker's cell. */
-	update_cell_display(other->side, ux, uy, TRUE);
+	update_cell_display(other->side, ux, uy, UPDATE_ALWAYS);
 	/* Actually, everybody might be seeing the combat. */
 	all_see_cell(ux, uy);
 	all_see_cell(ox, oy);
@@ -1523,7 +1522,7 @@ int captype;
 
     /* Update everybody's view of the situation. */
     see_exact(ps, px, py);
-    update_cell_display(ps, px, py, TRUE);
+    update_cell_display(ps, px, py, UPDATE_ALWAYS);
     all_see_cell(px, py);
 }
 
@@ -1590,8 +1589,6 @@ Side *prevside;
       pris->act->nextaction.type = ACTION_NONE;
     /* (should probably adjust side's acp sums by new unit's amounts) */
     init_unit_plan(pris);
-    /* Assign a new number to this unit if it should have one. */
-    assign_unit_number(pris);
     /* Maybe get a pile of interesting information. */
     if (!all_see_all
 	&& unit->side
@@ -1605,7 +1602,7 @@ Side *prevside;
 		    && terrain_view(unit->side, x, y) == UNSEEN) {
 		    set_terrain_view(unit->side, x, y, terrain_view(prevside, x, y));
 		    /* (should also view see-always units here) */
-		    update_cell_display(unit->side, x, y, FALSE);
+		    update_cell_display(unit->side, x, y, UPDATE_ALWAYS);
 		    notesee = TRUE;
 		}
 	    }
@@ -1616,7 +1613,7 @@ Side *prevside;
 		    chance = uu_see_others_captured(pris->type, unit2->type);
 		    if (probability(chance)) {
 			see_exact(unit->side, unit2->x, unit2->y);
-			update_cell_display(unit->side, unit2->x, unit2->y, TRUE);
+			update_cell_display(unit->side, unit2->x, unit2->y, UPDATE_ALWAYS);
 			notesee = TRUE;
 		    }
 		}
