@@ -1,5 +1,5 @@
 /* Main include file for Xconq.
-   Copyright (C) 1991-1998 Stanley T. Shebs.
+   Copyright (C) 1991-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -29,11 +29,12 @@ any later version.  See the file COPYING.  */
 
 /* Elements of bitmask controlling display updates. */
 
-#define UPDATE_RIGHTNOW 1
-#define UPDATE_COVER 2
-#define UPDATE_TEMP 4
-#define UPDATE_CLOUDS 8
-#define UPDATE_WINDS 16
+#define UPDATE_ALWAYS 1
+#define UPDATE_ADJ 2
+#define UPDATE_COVER 4
+#define UPDATE_TEMP 8
+#define UPDATE_CLOUDS 16
+#define UPDATE_WINDS 32
 
 /* Declarations of globally visible globals. */
 
@@ -229,8 +230,8 @@ extern int unit_speed PARAMS ((Unit *unit, int nx, int ny));
 extern int zoc_range PARAMS ((Unit *unit, int u2));
 extern int damaged_value PARAMS ((Unit *unit, Obj *effect, int maxval));
 
-extern int can_research PARAMS ((Unit *unit));
-extern int type_can_research PARAMS ((int u));
+extern int can_develop PARAMS ((Unit *unit));
+extern int type_can_develop PARAMS ((int u));
 extern int can_toolup PARAMS ((Unit *unit));
 extern int type_can_toolup PARAMS ((int u));
 extern int can_create PARAMS ((Unit *unit));
@@ -298,7 +299,7 @@ extern void kick_out_enemy_users PARAMS ((Side *side, int x, int y));
 
 /* run2.c */
 
-extern void update_cell_display_all_sides PARAMS ((int x, int y, int rightnow));
+extern void update_cell_display_all_sides PARAMS ((int x, int y, int flags));
 extern void update_side_display_all_sides PARAMS ((Side *side, int rightnow));
 
 /* mkterr.c */
@@ -367,7 +368,7 @@ extern void acp_desc PARAMS ((char *buf, Unit *unit, int label));
 extern void cxp_desc PARAMS ((char *buf, Unit *unit, int label));
 extern void morale_desc PARAMS ((char *buf, Unit *unit, int label));
 extern void plan_desc PARAMS ((char *buf, Unit *unit));
-extern void task_desc PARAMS ((char *buf, Side *side, Task *task));
+extern void task_desc PARAMS ((char *buf, Side *side, Unit *unit, Task *task));
 extern void time_desc PARAMS ((char *buf, int time, int maxtime));
 extern char *summarize_units PARAMS ((char *buf, int *ucnts));
 extern void notify_doctrine PARAMS ((Side *side, char *spec));
@@ -392,7 +393,7 @@ extern int short_side_title_plural_p PARAMS ((Side *side));
 extern char *short_side_title_with_adjective PARAMS ((Side *side, char *adjective));
 extern char *side_score_desc PARAMS ((char *buf, Side *side, Scorekeeper *sk));
 extern void location_desc PARAMS ((char *buf, Side *side, Unit *unit, int u, int x, int y));
-extern void destination_desc PARAMS ((char *buf, Side *side, int x, int y, int z));
+extern void destination_desc PARAMS ((char *buf, Side *side, Unit *unit, int x, int y, int z));
 extern int supply_desc PARAMS ((char *buf, Unit *unit, int mrow));
 extern char *sidemask_desc PARAMS ((char *buf, SideMask sidemask));
 extern void latlong_desc PARAMS ((char *buf, int x, int y, int xf, int yf, int which));
