@@ -1,5 +1,5 @@
 /* General Xconq initialization.
-   Copyright (C) 1987-1989, 1991-1998 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@ any later version.  See the file COPYING.  */
 #include "conq.h"
 #include "kernel.h"
 #include "version.h"
+extern int gameinited;
 
 static void cache_possible_types PARAMS ((void));
 static void kill_excess_units PARAMS ((void));
@@ -255,7 +256,7 @@ void
 check_game_validity()
 {
     int failed = FALSE, movers = FALSE, actors = FALSE;
-    int u1, u2, m1, t1, t2, i, methkey, found;
+    int u1, u2, m1, t1, t2, a1, i, methkey, found;
     Obj *synthlist = g_synth_methods(), *methods, *method;
 
     /* We must have at least one kind of unit. */
@@ -377,6 +378,14 @@ check_game_validity()
 	    failed = TRUE;
 	}
     }
+    /* Check various advance type properties. */
+    for_all_advance_types(a1) {
+	/* Can't make use of this yet, so error out if anybody tries. */
+	if (a_available(a1) != 1) {
+	    init_error("advance type %d not always available", a1);
+	    failed = TRUE;
+	}
+    }
     /* If nothing can move and nothing can build, this will probably be
        a really dull game, but there may be such games, do don't say
        anything normally. */
@@ -588,9 +597,7 @@ patch_object_references()
 		  unit->act->acp = u_acp(unit->type);
 		/* Restore initacp that wasn't written out because it
 		   was the normal value. */
-		if (unit->act
-		    && unit->act->acp > 0
-		    && unit->act->initacp == 0)
+		if (unit->act && unit->act->initacp < u_acp_min(unit->type))
 		  unit->act->initacp = u_acp(unit->type);
 		/* Might already have a plan, leave alone if so. */
 		if (unit->plan == NULL) {
@@ -877,9 +884,34 @@ remove_side_and_player()
 void
 run_synth_methods()
 {
-    int i, methkey, found = FALSE, rslt;
-    Obj *synthlist, *methods, *method, *parms;
+    int i, methkey, found, rslt;
+    Obj *restlist, *done, *synthlist, *methods, *method, *parms, *donelist;
 
+    /* Restore any saved state for synthmethods. */
+    if (!gameinited) {
+	for_all_list(g_synth_done(), restlist) {
+	    done = car(restlist);
+	    method = car(done);
+	    found = FALSE;
+	    if (symbolp(method)) {
+		methkey = keyword_code(c_string(method));
+		for (i = 0; synthmethods[i].key >= 0; ++i) {
+		    if (methkey == synthmethods[i].key) {
+			found = TRUE;
+			synthmethods[i].calls = c_number(cadr(done));
+			synthmethods[i].runs = c_number(caddr(done));
+			break;
+		    }
+		}
+	    } else {
+		init_warning("non-symbol in synthesis list");
+	    }
+	    if (!found) {
+		sprintlisp(spbuf, method, BUFSIZE);
+		init_warning("bad synthesis method %s, ignoring", spbuf);
+	    }
+	}
+    }
     /* Make sure no dead units get saved. */
     flush_dead_units();
     /* Make a consistent ordering of units. */
@@ -888,35 +920,39 @@ run_synth_methods()
     Dprintf("Will run syntheses ");
     Dprintlisp(synthlist);
     Dprintf("\n");
+    donelist = lispnil;
     for_all_list(synthlist, methods) {
 	method = car(methods);
+	found = FALSE;
 	if (symbolp(method)) {
 	    methkey = keyword_code(c_string(method));
 	    for (i = 0; synthmethods[i].key >= 0; ++i) {
 		if (methkey == synthmethods[i].key) {
 		    found = TRUE;
-		    rslt = (*synthmethods[i].fn)(synthmethods[i].calls, synthmethods[i].runs);
-		    ++(synthmethods[i].calls);
-		    if (rslt)
-		      ++(synthmethods[i].runs);
+		    if (!gameinited && synthmethods[i].calls == 0) {
+			rslt = (*synthmethods[i].fn)(synthmethods[i].calls,
+						     synthmethods[i].runs);
+			++(synthmethods[i].calls);
+			if (rslt)
+			  ++(synthmethods[i].runs);
+		    }
 		    break;
 		}
 	    }
-	} else if (consp(method)) {
-	    parms = cdr(method);
-	    if (stringp(car(method))) {
-		/* External program. */
-		/* (should format parms, compose outputfile redirection,
-		    call program with os.c function, check return code,
-		    open and read output file) */
-		run_error("No external synth programs yet");
-	    }
+	    donelist = cons(cons(method,
+				 cons(new_number(synthmethods[i].calls),
+				      cons(new_number(synthmethods[i].runs),
+					   lispnil))),
+			    donelist);
+	} else {
+	    init_warning("non-symbol in synthesis list");
 	}
 	if (!found) {
 	    sprintlisp(spbuf, method, BUFSIZE);
 	    init_warning("bad synthesis method %s, ignoring", spbuf);
 	}
     }
+    set_g_synth_done(reverse(donelist));
 }
 
 int
@@ -1180,11 +1216,11 @@ int x, y;
 	    	return;
 	    }
 	    chance = already_seen_chance(tmpside, unit);
-	    if (probability(chance)) {
+	    if (!tmpside->unit_view_restored && probability(chance)) {
 	    	see_exact(tmpside, x, y);
-	    	/* This view might be overwritten by a view of
-	    	   another might-be-seen unit at this location,
-	    	   so don't return just yet. */
+	    	/* This view might be overwritten by a view of another
+	    	   might-be-seen unit at this location, so don't
+	    	   return just yet. */
 	    }
 	}
     }
@@ -1330,7 +1366,7 @@ int
 make_initial_materials(calls, runs)
 int calls, runs;
 {
-    int m, t, amts[MAXTTYPES], doany, x, y;
+    int m, t, amts[MAXTTYPES], doany, x, y, didany = FALSE;
 
     /* Go over each material and terrain type, looking for nonzero
        material in terrain possibilities, then alloc and fill in layers
@@ -1348,9 +1384,10 @@ int calls, runs;
 		t = terrain_at(x, y);
 		set_material_at(x, y, m, amts[t]);
 	    }
+	    didany = TRUE;
 	}
     }
-    return TRUE;
+    return didany;
 }
 
 /* Give the unit what it is declared to have stockpiled
