@@ -428,7 +428,7 @@ make_random_terrain(calls, runs)
 int calls, runs;
 {
     int t, x, y, cellsum, cellsumtable[MAXTTYPES];
-    int i, numcells, n, elevrange, occur, dir;
+    int i, numcells, n, elevrange, elev, occur, dir;
 
     if (terrain_defined())
       return FALSE;
@@ -458,7 +458,8 @@ int calls, runs;
 	set_terrain_at(x, y, t);
 	if (elevations_defined() && !world_is_flat()) {
 	    elevrange = t_elev_max(t) - t_elev_min(t) + 1;
-	    set_elev_at(x, y, xrandom(elevrange) + t_elev_min(t));
+	    elev = xrandom(elevrange) + t_elev_min(t);
+	    set_elev_at(x, y, elev);
 	}
     }
     /* Make sure the edge of the area has something in it. */
@@ -1416,8 +1417,9 @@ fix_adjacent_terrain()
 	for_all_terrain_types(t2) {
 	    t3 = tt_adj_terr_effect(t1, t2);
 	    if (t3 >= 0) {
-		/* Table definitions have no provision for dynamically-determined
-		   bounds, so we have to check it here. */
+		/* Table definitions have no provision for
+		   dynamically-determined bounds, so we have to check
+		   it here. */
 		if (!is_terrain_type(t3)) {
 		    run_warning("Bad value %d in adjacent-terrain-effect table, ignoring", t3);
 		    break;
