@@ -555,7 +555,7 @@ int nx, ny;
     /* Use up supplies as directed. */
     consume_move_supplies(unit);
     /* a hack */
-    update_cell_display(unit->side, ox, oy, TRUE);
+    update_cell_display(unit->side, ox, oy, UPDATE_ALWAYS);
     /* Always return the mp cost, even if the mover died. */
     return total_move_cost(u, u3, ox, oy, oz, nx, ny, nz);
 }
@@ -580,19 +580,14 @@ Side *side;
 		}
 	    }
 	}
-	update_cell_display_all_sides(x, y, TRUE);
-	for_all_directions(dir) {
-	    if (interior_point_in_dir(x, y, dir, &x1, &y1)) {
-		update_cell_display_all_sides(x1, y1, TRUE);
-	    }
-	}
+	update_cell_display_all_sides(x, y, UPDATE_ALWAYS | UPDATE_ADJ);
 	/* Previous side(s) lose free coverage. */
 	for_all_sides(side2) {
 	    if (!trusted_side(side, side2)
 		&& (oldside == side2 || trusted_side(oldside, side2))) {
 		add_cover(side2, x, y, -1);
 		/* Update coverage display. */
-		update_cell_display(side2, x, y, 36);
+		update_cell_display(side2, x, y, UPDATE_COVER);
 	   }
 	}
     }
