@@ -82,14 +82,14 @@ typedef struct a_strategy {
     short *actualmix;
     short *expectedmix;
     short *idealmix;
-    short *research_status;  /* specific to the "time" game */
-    short *research_on;      /* specific to the "time" game */
+    short *develop_status;  /* specific to the "time" game */
+    short *develop_on;      /* specific to the "time" game */
     Obj *writable_state;
 } Strategy;
 
-/* utype-specific research status codes for the "time" game */
-#define RS_RESEARCH_NEEDED 4
-#define RS_RESEARCH_ASSIGNED 3
+/* utype-specific develop status codes for the "time" game */
+#define RS_DEVELOP_NEEDED 4
+#define RS_DEVELOP_ASSIGNED 3
 #define RS_UPGRADE_NEEDED 1
 
 #define mplayer(s) ((Strategy *) (s)->ai)
@@ -196,9 +196,9 @@ static int preferred_build_type(Side *side, Unit *unit, int plantype);
 static int need_more_transportation(Side *side);
 static void assign_to_defense(Side *side, Unit *unit);
 static void assign_to_defense_support(Side *side, Unit *unit);
-static int assign_to_research_on(Side *side, Unit *unit, int u2);
-static int can_research_on(int u, int u2);
-static int needs_research(Side *side, int u);
+static int assign_to_develop_on(Side *side, Unit *unit, int u2);
+static int can_develop_on(int u, int u2);
+static int needs_develop(Side *side, int u);
 static int mplayer_guide_explorer(Side *side, Unit *unit);
 static int probably_explorable(Side *side, int x, int y, int u);
 static int build_base_for_self(Side *side, Unit *unit);
@@ -348,13 +348,13 @@ Side *side;
     /* code specific to the "time" game */
     if (game_class == gc_time) {
 	for_all_unit_types(u) {
-	    if (mplayer(side)->research_status[u] == RS_RESEARCH_ASSIGNED) {
-		u2 = mplayer(side)->research_on[u];
-		if (!needs_research (side, u2)) {
-		    /* research done, start upgrading */
-		    DMprintf("%s has completed research on %s\n",
+	    if (mplayer(side)->develop_status[u] == RS_DEVELOP_ASSIGNED) {
+		u2 = mplayer(side)->develop_on[u];
+		if (!needs_develop (side, u2)) {
+		    /* develop done, start upgrading */
+		    DMprintf("%s has completed develop on %s\n",
 			     side_desig(side), u_type_name(u2));
-		    mplayer(side)->research_status[u] = RS_UPGRADE_NEEDED;
+		    mplayer(side)->develop_status[u] = RS_UPGRADE_NEEDED;
 		}
 	    }
 	}
@@ -407,8 +407,8 @@ Side *side;
     strategy->actualmix = (short *) xmalloc(numutypes * sizeof(short));
     strategy->expectedmix = (short *) xmalloc(numutypes * sizeof(short));
     strategy->idealmix = (short *) xmalloc(numutypes * sizeof(short));
-    strategy->research_status = (short *) xmalloc(numutypes * sizeof(short));
-    strategy->research_on     = (short *) xmalloc(numutypes * sizeof(short));
+    strategy->develop_status = (short *) xmalloc(numutypes * sizeof(short));
+    strategy->develop_on     = (short *) xmalloc(numutypes * sizeof(short));
     /* Arrays for terrain types. */
     strategy->terrainguess = (short *) xmalloc(numttypes * sizeof(short));
     strategy->writable_state = lispnil;
@@ -446,16 +446,16 @@ Side *side;
 	strategy->actualmix[u] = 0;
 	strategy->expectedmix[u] = 0;
 	strategy->idealmix[u] = 0;
-	strategy->research_status[u] = 0;
-	strategy->research_on[u] = 0;
+	strategy->develop_status[u] = 0;
+	strategy->develop_on[u] = 0;
 
 	/* code specific to the "time" game */
 	if (game_class == gc_time) {
 	    for_all_unit_types(u2) {
-		if (needs_research (side, u2) && can_research_on(u, u2)) {
-		    strategy->research_status[u] = RS_RESEARCH_NEEDED;
-		    strategy->research_on[u] = u2;
-		    DMprintf("%s can research on %s (to level %d)\n",
+		if (needs_develop (side, u2) && can_develop_on(u, u2)) {
+		    strategy->develop_status[u] = RS_DEVELOP_NEEDED;
+		    strategy->develop_on[u] = u2;
+		    DMprintf("%s can develop on %s (to level %d)\n",
 			     u_type_name(u), u_type_name(u2),
 			     u_tech_to_build(u2));
 		}
@@ -1382,34 +1382,34 @@ Side *side;
     /* This code is specific to the "time" game. */
     if (game_class == gc_time) {
 	for_all_unit_types(u) {
-	    u2 = mplayer(side)->research_on[u];
-	    if (mplayer(side)->research_status[u] == RS_RESEARCH_ASSIGNED) {
-		/* is anyone researching? */
+	    u2 = mplayer(side)->develop_on[u];
+	    if (mplayer(side)->develop_status[u] == RS_DEVELOP_ASSIGNED) {
+		/* is anyone developing? */
 		unit2 = NULL;
 		for_all_side_units(side, unit) {
 		    if (unit->type==u && in_play(unit) &&
 			unit->plan && unit->plan->aicontrol) {
 			if (unit->plan->tasks &&
-			    unit->plan->tasks->type == TASK_RESEARCH &&
+			    unit->plan->tasks->type == TASK_DEVELOP &&
 			    unit->plan->tasks->args[0] == u2) 
 			  unit2 = unit;
 		    }
 		}
 		if (unit2 != NULL) {
-		    DMprintf("%s is researching for %s on %s (level %d/%d)\n",
+		    DMprintf("%s is developing for %s on %s (level %d/%d)\n",
 			     unit_desig(unit2), side_desig(side),
 			     u_type_name(u2),
 			     side->tech[u2], u_tech_to_build(u2));
 		} else {
-		    DMprintf("no %s is researching for %s on %s!\n",
+		    DMprintf("no %s is developing for %s on %s!\n",
 			     u_type_name(u), side_desig(side),
 			     u_type_name(u2));
-		    mplayer(side)->research_status[u] = RS_RESEARCH_NEEDED;
+		    mplayer(side)->develop_status[u] = RS_DEVELOP_NEEDED;
 		}
 	    }
-	    if (mplayer(side)->research_status[u] == RS_RESEARCH_NEEDED
-		&& needs_research (side, u2)) {
-		/* pick for research a unit not building; 
+	    if (mplayer(side)->develop_status[u] == RS_DEVELOP_NEEDED
+		&& needs_develop (side, u2)) {
+		/* pick for develop a unit not building; 
 		   if all are building, choose the one which started last */
 		unit2 = NULL;
 		cpmin = 9999;
@@ -1438,13 +1438,13 @@ Side *side;
 		}
 		if (unit2 == NULL) {
 		    if (any)
-		      DMprintf("no %s is available to research for %s on %s!\n",
+		      DMprintf("no %s is available to develop for %s on %s!\n",
 			       u_type_name(u), side_desig(side),
 			       u_type_name(u2));
 		} else {
-		    if (assign_to_research_on(side, unit2, u2)) {
-			mplayer(side)->research_status[u] =
-			  RS_RESEARCH_ASSIGNED;
+		    if (assign_to_develop_on(side, unit2, u2)) {
+			mplayer(side)->develop_status[u] =
+			  RS_DEVELOP_ASSIGNED;
 		    }
 		}
 	    }
@@ -1476,10 +1476,10 @@ Side *side;
 	    /* code specific to the "time" game */
 	    if (game_class == gc_time) {
 		u = unit->type;
-		u2 = mplayer(side)->research_on[u];
+		u2 = mplayer(side)->develop_on[u];
 
 		/* should we upgrade? */
-		if (mplayer(side)->research_status[u] == RS_UPGRADE_NEEDED) {
+		if (mplayer(side)->develop_status[u] == RS_UPGRADE_NEEDED) {
 		    cp = 0;
 		    occ = NULL;
 		    if ((unit->plan->tasks != NULL &&
@@ -1723,15 +1723,15 @@ Unit *unit;
     if (plan == NULL || !plan->aicontrol)
       return;
     /* code specific to the "time" game */
-    /* don't mess up with units researching or upgrading */
+    /* don't mess up with units developing or upgrading */
     if (game_class == gc_time) {
-	if (mplayer(side)->research_status[u] == RS_UPGRADE_NEEDED
+	if (mplayer(side)->develop_status[u] == RS_UPGRADE_NEEDED
 	    && plan->tasks != NULL
 	    && plan->tasks->type == TASK_BUILD)
 	  return;
-	if (mplayer(side)->research_status[u] == RS_RESEARCH_ASSIGNED
+	if (mplayer(side)->develop_status[u] == RS_DEVELOP_ASSIGNED
 	    && plan->tasks != NULL
-	    && plan->tasks->type == TASK_RESEARCH)
+	    && plan->tasks->type == TASK_DEVELOP)
 	  return;
     }
     /* If we're not trying to win at this game, then make the unit act randomly. */
@@ -1940,6 +1940,9 @@ Theater *theater;
 		  + xrandom(theater->xmax - theater->xmin)) / 2;
 	    y += (xrandom(theater->ymax - theater->ymin)
 		  + xrandom(theater->ymax - theater->ymin)) / 2;
+	    /* Filter out invalid locations. */
+	    if (!inside_area(x, y))
+	      continue;
 	    if (theater_at(side, x, y) == theater
 		&& terrain_view(side, x, y) == UNSEEN
 		&& probably_explorable(side, x, y, unit->type))
@@ -2197,7 +2200,7 @@ int plantype;
 	prefs[u2] = 0;
 	est = est_completion_time(unit, u2);
 	if (could_create(u, u2)
-	    /* tmp hack until mplayer can do research */
+	    /* tmp hack until mplayer can do develop */
 	    && (u_tech_to_build(u2) > 0 ? side->tech[u2] >= u_tech_to_build(u2) : TRUE)
 	    && est >= 0
 	    && type_allowed_on_side(u2, side)) {
@@ -2387,11 +2390,11 @@ Unit *unit;
 }
 
 static int
-can_research_on(u, u2)
+can_develop_on(u, u2)
 int u;
 int u2;
 {
-    int acp_res = uu_acp_to_research(u, u2);
+    int acp_res = uu_acp_to_develop(u, u2);
 
     if (acp_res < 1 || acp_res > u_acp(u))
       return 0;
@@ -2401,7 +2404,7 @@ int u2;
 
 /* code specific to the "time" game */
 static int 
-needs_research (side, u)
+needs_develop (side, u)
 Side *side;
 int u;
 {
@@ -2425,24 +2428,24 @@ int u;
 }
 
 static int
-assign_to_research_on(side, unit, u2)
+assign_to_develop_on(side, unit, u2)
 Side *side;
 Unit *unit;
 int u2;
 {
     int lev = u_tech_to_build(u2);
 
-    if (!can_research_on(unit->type, u2)) {
-	DMprintf("%s cannot research on %s!\n",
+    if (!can_develop_on(unit->type, u2)) {
+	DMprintf("%s cannot develop on %s!\n",
 	     unit_desig(unit), u_type_name(u2));
 	return 0;
     }
 
-    DMprintf("%s will research for %s on %s (to level %d)\n",
+    DMprintf("%s will develop for %s on %s (to level %d)\n",
 	     unit_desig(unit), side_desig(side), u_type_name(u2), lev);
 
     net_set_unit_plan_type(side, unit, PLAN_PASSIVE);
-    net_push_research_task(unit, u2, lev);
+    net_push_develop_task(unit, u2, lev);
     return 1;
 }
 
