@@ -1,5 +1,5 @@
 /* Interface-independent natural language handling for Xconq.
-   Copyright (C) 1987-1989, 1991-1998 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -766,7 +766,7 @@ int u;
 		tprintf(buf, "%d/%d done ", unit2->cp, u_cp(unit2->type));
 	    }
 	    tprintf(buf, "(%d of %d)", task->args[2] + 1, task->args[3]);
-	} else if (task->type == TASK_RESEARCH) {
+	} else if (task->type == TASK_DEVELOP) {
 	    u2 = task->args[0];
 	    if (is_unit_type(u2)) {
 		tprintf(buf, " %s tech %d/%d",
@@ -1729,41 +1729,43 @@ int u, x, y;
 }
 
 /* Given a cell, describe where it is. */
-/* (should also have a destination_desc_relative) */
 
 void
-destination_desc(buf, side, x, y, z)
-char *buf;
-Side *side;
-int x, y, z;
+destination_desc(char *buf, Side *side, Unit *unit, int x, int y, int z)
 {
     int dir, x1, y1;
-    Unit *unit;
+    Unit *unit2;
 
     if (!in_area(x, y)) {
 	sprintf(buf, "?%d,%d?", x, y);
 	return;
     }
-    unit = unit_at(x, y);
-    if (unit != NULL && unit->side == side) {
-	if (!empty_string(unit->name)) {
-	    sprintf(buf, "%s (%d,%d)", unit->name, x, y);
+    unit2 = unit_at(x, y);
+    if (unit2 != NULL && unit2->side == side) {
+	if (unit2 == unit) {
+	    sprintf(buf, "self (%d,%d)", x, y);
+	} else if (!empty_string(unit2->name)) {
+	    sprintf(buf, "%s (%d,%d)", unit2->name, x, y);
 	} else {
-	    sprintf(buf, "%s (%d,%d)", u_type_name(unit->type), x, y);
+	    sprintf(buf, "%s (%d,%d)", u_type_name(unit2->type), x, y);
 	}
 	return;
     }
     for_all_directions(dir) {
 	if (interior_point_in_dir(x, y, dir, &x1, &y1)) {
-	    unit = unit_at(x1, y1);
-	    if (unit != NULL && unit->side == side) {
-		if (!empty_string(unit->name)) {
+	    unit2 = unit_at(x1, y1);
+	    /* (should look at stack) */
+	    if (unit2 != NULL && unit2->side == side) {
+		if (unit2 == unit) {
+		    sprintf(buf, "%s (%d,%d)",
+			    dirnames[opposite_dir(dir)], x, y);
+		} else if (!empty_string(unit2->name)) {
 		    sprintf(buf, "%s of %s (%d,%d)",
-			    dirnames[opposite_dir(dir)], unit->name, x, y);
+			    dirnames[opposite_dir(dir)], unit2->name, x, y);
 		} else {
 		    sprintf(buf, "%s of %s (%d,%d)",
 			    dirnames[opposite_dir(dir)],
-			    u_type_name(unit->type), x, y);
+			    u_type_name(unit2->type), x, y);
 		}
 		return;
 	    }
@@ -1935,67 +1937,101 @@ Unit *unit;
 /* Describe a task in a brief but intelligible way. */
 
 void
-task_desc(buf, side, task)
-char *buf;
-Side *side;
-Task *task;
+task_desc(char *buf, Side *side, Unit *unit, Task *task)
 {
-    int i, slen;
+    int i, slen, arg0, arg1, arg2, arg3;
     char *argtypes;
-    Unit *unit;
+    Unit *unit2;
 
     if (task == NULL) {
 	buf[0] = '\0';
 	return;
     }
-    sprintf(buf, "%s", taskdefns[task->type].name);
+    arg0 = task->args[0];
+    arg1 = task->args[1];
+    arg2 = task->args[2];
+    arg3 = task->args[3];
     switch (task->type) {
       case TASK_BUILD:
-	tprintf(buf, " %s", u_type_name(task->args[0]));
-	if (task->args[1] != 0) {
-	    unit = find_unit(task->args[1]);
-	    if (unit != NULL) {
-		tprintf(buf, " (%d cp)", unit->cp);
+	sprintf(buf, "build %s, ", u_type_name(arg0));
+	if (arg1 != 0) {
+	    unit2 = find_unit(arg1);
+	    if (unit2 != NULL) {
+		tprintf(buf, "%d/%d done with", unit2->cp, u_cp(unit2->type));
 	    }
+	} else {
+	    tprintf(buf, "to start on");
+	}
+	tprintf(buf, " %d%s of %d", arg2 + 1, ordinal_suffix(arg2 + 1), arg3);
+	break;
+      case TASK_COLLECT:
+	sprintf(buf, "collect %s around ", m_type_name(arg0));
+        destination_desc(buf+strlen(buf), side, unit, arg1, arg2, 0);
+	break;
+      case TASK_DEVELOP:
+	sprintf(buf, "develop %s to tech %d", u_type_name(arg0), arg1);
+	if (side != NULL) {
+	    tprintf(buf, " (currently at %d)", side->tech[arg0]);
 	}
-	tprintf(buf, ", %d of %d", task->args[2], task->args[3]);
 	break;
       case TASK_HIT_POSITION:
-	tprintf(buf, " ");
-        destination_desc(buf+strlen(buf), side, task->args[0], task->args[1], 0);
+	sprintf(buf, "hit position ");
+        destination_desc(buf+strlen(buf), side, unit, arg0, arg1, 0);
 	break;
       case TASK_HIT_UNIT:
-	tprintf(buf, " at ");
-        destination_desc(buf+strlen(buf), side, task->args[0], task->args[1], 0);
-	if (task->args[2] != NONUTYPE) {
-	    tprintf(buf, " (s%d %s)", task->args[3], shortest_unique_name(task->args[2]));
+	sprintf(buf, "hit unit at ");
+        destination_desc(buf+strlen(buf), side, unit, arg0, arg1, 0);
+	if (arg2 != NONUTYPE) {
+	    tprintf(buf, " (%s %s)",
+		    side_adjective(side_n(arg3)), shortest_unique_name(arg2));
       	}
 	break;
       case TASK_MOVE_DIR:
-	tprintf(buf, " %s %d", dirnames[task->args[0]], task->args[1]);
+	sprintf(buf, "move %s, %d times", dirnames[arg0], arg1);
 	break;
       case TASK_MOVE_TO:
-        if (task->args[3] == 0) {
+	sprintf(buf, "move to");
+	if (unit != NULL && unit->x == arg0 && unit->y == arg1) {
+	    tprintf(buf, " here");
+	    break;
+        } else if (arg3 == 0) {
 	    tprintf(buf, " ");
-        } else if (task->args[3] == 1) {
+        } else if (arg3 == 1) {
 	    tprintf(buf, " adj ");
         } else {
-	    tprintf(buf, " within %d of ", task->args[3]);
+	    tprintf(buf, " within %d of ", arg3);
         }
-        destination_desc(buf+strlen(buf), side, task->args[0], task->args[1], task->args[2]);
+        destination_desc(buf+strlen(buf), side, unit, arg0, arg1, arg2);
+	break;
+      case TASK_OCCUPY:
+	sprintf(buf, "occupy ");
+	unit2 = find_unit(arg0);
+	if (unit2 != NULL) {
+	    tprintf(buf, "%s", unit_handle(side, unit2));
+	} else {
+	    tprintf(buf, "unknown unit #%d", arg0);
+	}
 	break;
       case TASK_RESUPPLY:
-	if (task->args[0] != NONMTYPE)
-	  tprintf(buf, " %s", m_type_name(task->args[0]));
-	if (task->args[1] != 0) {
-	    unit = find_unit(task->args[1]);
-	    if (unit != NULL) {
-		tprintf(buf, " at %s", short_unit_handle(unit));
+	sprintf(buf, "%s", taskdefns[task->type].name);
+	if (arg0 != NONMTYPE)
+	  tprintf(buf, " %s", m_type_name(arg0));
+	else
+	  tprintf(buf, " all");
+	if (arg1 != 0) {
+	    unit2 = find_unit(arg1);
+	    if (unit2 != NULL) {
+		tprintf(buf, " at %s", short_unit_handle(unit2));
 	    }
 	}
 	break;
+      case TASK_SENTRY:
+	sprintf(buf, "sentry, for %d turns", arg0);
+	break;
       default:
-	/* Default is just to dump out the raw parameters of the task. */
+	/* Default is just to dump out the raw data about the task. */
+	sprintf(buf, "%s", taskdefns[task->type].name);
+	tprintf(buf, " raw");
 	argtypes = taskdefns[task->type].argtypes;
 	slen = strlen(argtypes);
 	for (i = 0; i < slen; ++i) {
@@ -2003,10 +2039,12 @@ Task *task;
 	}
 	break;
     }
-    /* Always (should we?) include the number of executions and retries. */
-    tprintf(buf, " x %d", task->execnum);
-    if (task->retrynum > 0) {
-	tprintf(buf, " fail %d", task->retrynum);
+    if (Debug) {
+	/* Include the number of executions and retries. */
+	tprintf(buf, " x %d", task->execnum);
+	if (task->retrynum > 0) {
+	    tprintf(buf, " fail %d", task->retrynum);
+	}
     }
 }
 
