@@ -599,8 +599,8 @@ Unit *unit;
 	    goal = occ->plan->maingoal;
 	    if (goal != NULL
 		&& goal->type == GOAL_VICINITY_HELD
-		&& distance(goal->args[0], goal->args[1], unit->x, unit->y)
-		> goal->args[2]) {
+		&& (distance(goal->args[0], goal->args[1], unit->x, unit->y)
+		    > goal->args[2])) {
 		set_move_to_task(unit, goal->args[0], goal->args[1],
 				  max(goal->args[2] / 2, 1));
 		DMprintf("%s will go where occupant %s wants to go (goal %s)\n",
@@ -615,8 +615,9 @@ Unit *unit;
 			|| task->args[1] != unit->y)
 			&& distance(task->args[0], task->args[1], unit->x, unit->y) > 1
 			) {
-		    /* Note that we assume the transport is mobile, which is OK currently
-		       because of where this routine is called from. */
+		    /* Note that we assume the transport is mobile,
+		       which is OK currently because of where this
+		       routine is called from. */
 		    set_move_to_task(unit, task->args[0], task->args[1], 1);
 		    DMprintf("%s will go where occupant %s wants to go (task %s)\n",
 			     unit_desig(unit), unit_desig(occ), task_desig(task));
@@ -1798,7 +1799,10 @@ plan_random(Unit *unit)
 	  case TASK_COLLECT:
 	    if (nummtypes > 0) {
 		randmtype = (flip_coin() ? xrandom(nummtypes) : NONMTYPE);
-	    	set_collect_task(unit, randmtype, unit->x, unit->y);
+		random_point_near(unit->x, unit->y, 4, &x1, &y1);
+	    	set_collect_task(unit, randmtype, x1, y1);
+	    } else {
+		set_sentry_task(unit, xrandom(5));
 	    }
 	    break;
 	  case TASK_DISBAND:
@@ -1810,6 +1814,11 @@ plan_random(Unit *unit)
 	    break;
 	  case TASK_HIT_POSITION:
 	    task = create_task(tasktype);
+	    if (flip_coin())
+	      random_point_near(unit->x, unit->y, 4, &x1, &y1);
+	    else
+	      random_point(&x1, &y1);
+	    task->args[0] = x1;  task->args[1] = y1;
 	    add_task(unit, CLEAR_AGENDA, task);
 	    break;
 	  case TASK_HIT_UNIT:
@@ -1839,12 +1848,14 @@ plan_random(Unit *unit)
 		randmtype = (flip_coin() ? xrandom(nummtypes) : NONMTYPE);
 		task = create_produce_task(unit, randmtype, xrandom(100));
 		add_task(unit, CLEAR_AGENDA, task);
+	    } else {
+		set_sentry_task(unit, xrandom(5));
 	    }
 	    break;
 	  case TASK_REPAIR:
 	    set_repair_task(unit);
 	    break;
-	  case TASK_RESEARCH:
+	  case TASK_DEVELOP:
 	    task = create_task(tasktype);
 	    add_task(unit, CLEAR_AGENDA, task);
 	    break;
@@ -1852,6 +1863,8 @@ plan_random(Unit *unit)
 	    if (nummtypes > 0) {
 		randmtype = (flip_coin() ? xrandom(nummtypes) : NONMTYPE);
 		set_resupply_task(unit, randmtype);
+	    } else {
+		set_sentry_task(unit, xrandom(5));
 	    }
 	    break;
 	  case TASK_SENTRY:
@@ -2218,20 +2231,20 @@ Unit *unit;
 int prod;
 {
     int schedule = 1 /* uu_make(unit->type, prod) */;
-    int u, research_delay = 0;
+    int u, develop_delay = 0;
 
     /* Add penalty (or unpenalty!) for first unit of a type. */
     /* is "counts" a reliable way to test? */
     if (unit->side->counts[prod] <= 1) {
-/*	research_delay = ((schedule * u_research(prod)) / 100);  */
+/*	develop_delay = ((schedule * u_develop(prod)) / 100);  */
 	for_all_unit_types(u) {
 	    if (unit->side->counts[u] > 1) {
-		research_delay -=
+		develop_delay -=
 		  (1 /*uu_make(unit->type, u)*/ * 
 		   uu_tech_crossover(prod, u)) / 100;
 	    }
-	    if (research_delay > 0) {
-		schedule += research_delay;
+	    if (develop_delay > 0) {
+		schedule += develop_delay;
 	    }
 	}
     }
@@ -2550,7 +2563,7 @@ Unit *unit;
     for_all_unit_types(u2) {
 	if (could_create(unit->type, u2)
 	    || uu_acp_to_build(unit->type, u2) > 0
-	    || uu_acp_to_research(unit->type, u2) > 0)
+	    || uu_acp_to_develop(unit->type, u2) > 0)
 	  return TRUE;
     }
     return FALSE;
