@@ -1,5 +1,5 @@
 /* The main simulation-running code in Xconq.
-   Copyright (C) 1986-1989, 1991-1998 Stanley T. Shebs.
+   Copyright (C) 1986-1989, 1991-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -273,9 +273,13 @@ int maxactions;
 		/* Possibly finish some sides' turns. */
 		for_all_sides(side) {
 		    if (!side->finishedturn
-			&& !units_still_acting(side)
-			&& side->autofinish
-			&& !is_designer(side)) {
+			&& ((!units_still_acting(side)
+			     && side->autofinish
+			     && !is_designer(side))
+			    /* Sides with no display or AI finish
+                               automatically. */
+			    || (!side_has_display(side)
+				&& !side_has_ai(side)))) {
 			Dprintf("run_game: %s auto-finishes.\n",
 				side_desig(side));
 			finish_turn(side);
@@ -422,7 +426,7 @@ all_sides_finished()
     return TRUE;
 }
 
-/* This is true when AIs should move more slowly. */
+/* This is true when AIs should move more quickly. */
 
 int
 all_human_only_sides_finished()
@@ -1262,7 +1266,7 @@ run_advanced_units()
 	    for_all_sides(side) {
 		if (side->see_all || side_sees_unit(side, unit) 
 		    || side_tracking_unit(side, unit)) {
-		    update_cell_display(side, unit->x, unit->y, TRUE);
+		    update_cell_display(side, unit->x, unit->y, UPDATE_ALWAYS);
 		}
 	    }
 	}
