@@ -1,5 +1,5 @@
 /* The main simulation-running code in Xconq, turn start and end calcs.
-   Copyright (C) 1986-1989, 1991-1998 Stanley T. Shebs.
+   Copyright (C) 1986-1989, 1991-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -509,9 +509,9 @@ run_tech_leakage()
     }
 }
 
-/* Remember each side's tech levels before it does any research actions
+/* Remember each side's tech levels before it does any develop actions
    during the turn.  This can be used to keep tech level from going up too
-   fast if the player has lots of units doing research. */
+   fast if the player has lots of units doing development. */
 
 static void
 cache_init_tech_levels()
@@ -1248,7 +1248,7 @@ int x, y;
     for_all_sides(side) {
 	if (all_see_all) {
 	    update_cell_display(side, x, y,
-				UPDATE_TEMP|UPDATE_CLOUDS|UPDATE_WINDS);
+				UPDATE_TEMP | UPDATE_CLOUDS | UPDATE_WINDS);
 	} else if (g_see_weather_always()
 	    ? (terrain_view(side, x, y) != UNSEEN)
 	    : (cover(side, x, y) > 0)) {
@@ -1587,7 +1587,7 @@ int force;
 	/* (should set new opinions and maybe morale) */
 	/* Give the previous side a last view of the situation. */
 	see_exact(oldside, ux, uy);
-	update_cell_display(oldside, ux, uy, TRUE);
+	update_cell_display(oldside, ux, uy, UPDATE_ALWAYS);
 	all_see_cell(ux, uy);
     }
 }
@@ -2322,12 +2322,7 @@ Side *side;
 		}
 	    }
 	}
-	update_cell_display_all_sides(x, y, TRUE);
-	for_all_directions(dir) {
-	    if (interior_point_in_dir(x, y, dir, &x1, &y1)) {
-		update_cell_display_all_sides(x1, y1, TRUE);
-	    }
-	}
+	update_cell_display_all_sides(x, y, UPDATE_ALWAYS | UPDATE_ADJ);
 	/* Previous side(s) lose free coverage. */
 	for_all_sides(side2) {
 	    if (!trusted_side(side, side2)
@@ -2391,8 +2386,8 @@ run_people_limits()
 }
 
 void
-update_cell_display_all_sides(x, y, rightnow)
-int x, y, rightnow;
+update_cell_display_all_sides(x, y, flags)
+int x, y, flags;
 {
     Side *side;
 
@@ -2400,7 +2395,7 @@ int x, y, rightnow;
 	/* (Testing for unit visibility is not quite right, but close
 	   enough for now.) */
 	if (side->ingame && units_visible(side, x, y)) {
-	    update_cell_display(side, x, y, rightnow);
+	    update_cell_display(side, x, y, flags);
 	}
     }
 }
