@@ -713,6 +713,10 @@ int why;
 	run_warning("losing to side not in game, ignoring side");
 	side2 = NULL;
     }
+    /* If there's a controlling side, it gets everything. */
+    if (side->controlled_by != NULL && side->controlled_by->ingame) {
+	side2 = side->controlled_by;
+    }
     if (side2 != NULL) {
 	/* Dispose of all of a side's units. */
 	for_all_units(unit) {
