@@ -1,5 +1,5 @@
 /* Sides in Xconq.
-   Copyright (C) 1987-1989, 1991-1998 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -11,6 +11,7 @@ any later version.  See the file COPYING.  */
 
 #include "conq.h"
 #include "kernel.h"
+extern int gameinited;
 
 #define checked_elev_at(x, y) (elevations_defined() ? elev_at(x, y) : 0)
 
@@ -194,13 +195,6 @@ create_side()
     if (nummtypes > 0) {
 	newside->treasury = (short *) xmalloc(nummtypes * sizeof(short));
     }
-
-// --- SUPPORT FOR TURNING OFF AI RESIGNATION ----------------------------------------
-
-    newside->ai_may_resign = g_ai_may_resign();
-
-// --------------------------------------------------------------------------
-
     /* Link in at the end of the list of sides. */
     newside->next = NULL;
     lastside->next = newside;
@@ -286,7 +280,8 @@ Side *side;
     /* Be sure that we're not trying to set this up too soon. */
     check_area_shape();
     /* Allocate terrain view layers. */
-    if (side->terrview == NULL) {
+    if ((!g_terrain_seen() || !g_see_terrain_always())
+	&& side->terrview == NULL) {
 	side->terrview = malloc_area_layer(char);
 	if (!g_see_terrain_always()) {
 	    side->terrviewdate = malloc_area_layer(short);
@@ -1402,16 +1397,17 @@ int *types;
 	        && terrain_view(recipient, x, y) == UNSEEN) {
 	        set_terrain_view(recipient, x, y, terrain_view(sender, x, y));
 	        /* (should update unit views also) */
-		update_cell_display(recipient, x, y, TRUE);
 	    }
 	}
     }
     for_all_side_units(sender, unit) {
 	if (in_play(unit) && (types == NULL || types[unit->type])) {
 	    see_exact(recipient, unit->x, unit->y);
-	    update_cell_display(recipient, unit->x, unit->y, TRUE);
 	}
     }
+    /* Whether the amount of data is small or large, there is little value
+       in being selective about updates - just redo entire maps. */
+    update_area_display(recipient);
 }
 
 /* Modify doctrine according to a specification. */
@@ -1484,7 +1480,7 @@ int inopen;
     
     for_all_sides(side) {
 	if (side->see_all) {
-	    update_cell_display(side, x, y, TRUE);
+	    update_cell_display(side, x, y, UPDATE_ALWAYS);
 	} else if (side_sees_unit(side, unit)) {
 	    see_cell(side, x, y);
 	} else if (side_tracking_unit(side, unit)) {
@@ -1499,7 +1495,10 @@ int inopen;
 	    }
 	}
     }
-    {
+    /* If the occupation happens during the game, nearby units that
+       are on unfriendly sides should wake up.  (should also be:
+       "unless they were directed to sleep in presence of enemy") */
+    if (gameinited) {
 	int dir, x1, y1;
 	Unit *unit2, *unit3;
 	    	
@@ -1543,7 +1542,7 @@ int inopen;
 
     for_all_sides(side) {
 	if (side->see_all) {
-	    update_cell_display(side, x, y, TRUE);
+	    update_cell_display(side, x, y, UPDATE_ALWAYS);
 	} else if (side_sees_unit(side, unit)) {
 	    see_cell(side, x, y);
 	} else if (side_tracking_unit(side, unit)) {
@@ -1888,7 +1887,7 @@ int x0, y0, nx, ny;
 		    if ((newcov > 0 && oldcov == 0)
 			|| (newcov == 0 && oldcov > 0)
 			|| (DebugG && newcov != oldcov))
-		      update_cell_display(side, xw, y, 36);
+		      update_cell_display(side, xw, y, UPDATE_COVER);
 		    anychanges = TRUE;
 		}
 	    }
@@ -2230,17 +2229,17 @@ see_cell(side, x, y)
 Side *side;
 int x, y;
 {
-    int update, chance, t, curview, curuview, newuview, x1, y1;
-    int m, mupdate, wupdate, seentype, mistakechance;
+    int update, updatet, chance, t, curview, curuview, newuview, x1, y1;
+    int m, mupdate, wupdate, seentype, mistakechance, flags;
     Unit *unit, *unitseen, *seer;
 
     if (side == NULL || side == indepside || !in_area(x, y))
       return FALSE;
-    update = FALSE;
+    update = updatet = FALSE;
     unitseen = NULL;
     /* If we see everything, just pass through to updating the display. */
     if (side->see_all) {
-    	update = TRUE;
+    	update = updatet = TRUE;
     	unitseen = unit_at(x, y);
     } else if (cover(side, x, y) > 0) {
     	/* Always update our knowledge of the terrain. */
@@ -2249,7 +2248,7 @@ int x, y;
 	    set_terrain_view(side, x, y, buildtview(terrain_at(x, y)));
 	    if (!g_see_terrain_always())
 	      set_terrain_view_date(side, x, y, g_turn());
-	    update = TRUE;
+	    update = updatet = TRUE;
 	}
 	if (any_material_views) {
 	    mupdate = see_materials(side, x, y);
@@ -2353,12 +2352,19 @@ int x, y;
 	    set_unit_view(side, x, y, EMPTY);
 	    update = TRUE;
 	}
+#if 0 /* not used, and adds noise to savefiles */
 	/* In any case, our info is now up-to-date. */
 	set_unit_view_date(side, x, y, g_turn());
+#endif
     }
     /* If there was any change in what was visible, tell the display. */
     if (update) {
-	update_cell_display(side, x, y, FALSE);
+	flags = UPDATE_ALWAYS;
+	/* If the view of the terrain changed, adjacent cells probably
+	   need to be redrawn as well. */
+	if (updatet)
+	  flags |= UPDATE_ADJ;
+	update_cell_display(side, x, y, flags);
     }
     return (unitseen != NULL);
 }
@@ -2428,13 +2434,15 @@ int x, y;
 	if (unit != NULL)
 	  newuview = builduview(side_number(unit->side), unit->type);
 	set_unit_view(side, x, y, newuview);
+#if 0 /* not used, and adds noise to savefiles */
 	set_unit_view_date(side, x, y, g_turn());
+#endif
 	if (olduview != newuview)
 	  update = TRUE;
     }
     /* If there was any change in what was visible, tell the display. */
     if (update) {
-	update_cell_display(side, x, y, FALSE);
+	update_cell_display(side, x, y, UPDATE_ALWAYS | UPDATE_ADJ);
     }
 }
 
@@ -2938,7 +2946,7 @@ char *buf;
 	tprintf(buf, "%s %d", taskdefns[i].name, args[0]);
 	break;
     default:
-	task_desc(buf+strlen(buf), NULL, sorder->task);
+	task_desc(buf+strlen(buf), NULL, NULL, sorder->task);
 	break;
     }
     return buf;
@@ -3177,7 +3185,7 @@ int x, y;
     if (oldtview != tmptview || olduview != tmpuview) {
 	set_terrain_view(tmpside, x, y, tmptview);
 	set_unit_view(tmpside, x, y, tmpuview);
-	update_cell_display(tmpside, x, y, TRUE);
+	update_cell_display(tmpside, x, y, UPDATE_ALWAYS | UPDATE_ADJ);
     }
 }
 
