@@ -97,6 +97,9 @@ char *argv[];
     printf("(C) %s\n", copyright_string());
     init_library_path(NULL);
     clear_game_modules();
+#ifdef DEBUGGING
+    init_debug_to_stdout();
+#endif /* DEBUGGING */
     init_data_structures();
     /* Dump the random state so we can reproduce the run if necessary. */
     printf("Random state is %d", randstate);
@@ -763,16 +766,16 @@ update_area_display() {}
 /* (should count # of calls to inactive and non-displayed side...) */
 
 void
-update_cell_display(side, x, y, rightnow)
+update_cell_display(side, x, y, flags)
 Side *side;
 int x, y;
-int rightnow;
+int flags;
 {
     ++numcellupdatesperturn;
     if (active_display(side)) {
 	++numusefulcellupdatesperturn;
-	DGprintf("Update %s: view of %d,%d%s\n",
-		 side_desig(side), x, y, (rightnow ? " (now)" : ""));
+	DGprintf("Update %s: view of %d,%d 0x%x\n",
+		 side_desig(side), x, y, flags);
     }
 }
 
