@@ -1,5 +1,5 @@
 /* Unit task execution and general task functions.
-   Copyright (C) 1992-1998 Stanley T. Shebs.
+   Copyright (C) 1992-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -153,7 +153,7 @@ int x, y;
     return FALSE;
 }
 
-/* The build task handles the research, tooling up, creation, and completion
+/* The build task handles the develop, tooling up, creation, and completion
    for a given number of units of a given type. */
 
 static TaskOutcome
@@ -175,16 +175,11 @@ Task *task;
     if (is_unit_type(u2)
 	&& u_tech_to_build(u2) > 0
         && us->tech[u2] < u_tech_to_build(u2)) {
-        if (uu_acp_to_research(u, u2) > 0) {
-	    if (valid(check_research_action(unit, unit, u2))) {
-		prep_research_action(unit, unit, u2);
-		return TASK_PREPPED_ACTION;
-	    } else {
-		/* We get three trys to research before giving up. */
-	    	return (task->execnum < 3 ? TASK_IS_INCOMPLETE : TASK_FAILED);
-	    }
+        if (uu_acp_to_develop(u, u2) > 0) {
+	    push_develop_task(unit, u2, u_tech_to_build(u2));
+	    return TASK_IS_INCOMPLETE;
         } else {
-	    /* Can't do the necessary research. */
+	    /* Can't do the necessary development. */
 	    return TASK_FAILED;
         }
     }
@@ -305,11 +300,11 @@ Task *task;
     return NULL;
 }
 
-/* This is a "pure research" task, with the sole objective of increasing
-   technology. */
+/* This is a "pure development" task, with the sole objective of
+   increasing technology. */
 
 static TaskOutcome
-do_research_task(unit, task)
+do_develop_task(unit, task)
 Unit *unit;
 Task *task;
 {
@@ -317,20 +312,20 @@ Task *task;
     int u2 = task->args[0], lev = task->args[1];
     Side *us = unit->side;
 
-    /* Independents can never ever do research. */
+    /* Independents can never ever do development. */
     if (us == NULL)
       return TASK_FAILED;
     if (us->tech[u2] > u_tech_max(u2))
-      return TASK_FAILED; /* actually an error */
+      run_error("s%d tech for u%d is %d", side_number(us), u2, us->tech[u2]);
     if (us->tech[u2] >= lev)
       return TASK_IS_COMPLETE;
-    if (uu_acp_to_research(u, u2) <= 0)
+    if (uu_acp_to_develop(u, u2) <= 0)
       return TASK_FAILED;
-    if (valid(check_research_action(unit, unit, u2))) {
-	prep_research_action(unit, unit, u2);
+    if (valid(check_develop_action(unit, unit, u2))) {
+	prep_develop_action(unit, unit, u2);
 	return TASK_PREPPED_ACTION;
     } else {
-	/* We get three tries to research before giving up. */
+	/* We get three tries to develop before giving up. */
 	return (task->execnum < 3 ? TASK_IS_INCOMPLETE : TASK_FAILED);
     }
 }
@@ -1029,10 +1024,14 @@ Unit *unit;
 Task *task;
 {
     int m, tot, sofar, amt;
-    int x, y, dir, x1, y1, range;
-    Unit *collector;
+    int x, y, dir, x1, y1, x2, y2, range;
+    Unit *collector, *unit3;
 
     m = task->args[0];
+    x = task->args[1];  y = task->args[2];
+    /* Set up tmp globals early, all steps use these. */
+    tmpunit = unit;
+    tmpm = m;
     if (!is_material_type(m)) {
 	/* This may be an indication of code bogosity; warn? */
 	return TASK_FAILED;
@@ -1053,11 +1052,9 @@ Task *task;
 	/* Compute how far out to look for a delivery point; be a little
 	   optimistic. */
 	range = operating_range_best(unit->type);
-	tmpunit = unit;
-	tmpm = m;
-	if (search_around(unit->x, unit->y, range, collect_test, &x, &y, 1)) {
+	if (search_around(unit->x, unit->y, range, collect_test, &x2, &y2, 1)) {
     	    /* (should collect actual unit and chase it directly) */
-	    push_move_to_task(unit, x, y, 1);
+	    push_move_to_task(unit, x2, y2, 1);
 	    return TASK_IS_INCOMPLETE;
 	} else {
 	    /* Failure - sometimes just sit, but usually try something else. */
@@ -1067,10 +1064,20 @@ Task *task;
 	    return TASK_FAILED;
 	}
     }
-    for_all_directions(dir) {
-	if (interior_point_in_dir(unit->x, unit->y, dir, &x1, &y1)) {
-	    if (valid(check_extract_action(unit, unit, x1, y1, m, 1))) {
-		prep_extract_action(unit, unit, x1, y1, m, 1);
+    if (distance(unit->x, unit->y, x, y) > 1) {
+	push_move_to_task(unit, x, y, 1);
+	return TASK_IS_INCOMPLETE;
+    }
+    if (valid(check_extract_action(unit, unit, x, y, m, 1))) {
+	prep_extract_action(unit, unit, x, y, m, 1);
+	return TASK_PREPPED_ACTION;
+    }
+    for_all_stack(x, y, unit3) {
+	if (in_play(unit3)
+	    && unit3->side == unit->side
+	    && u_acp(unit3->type) == 0) {
+	    if (valid(check_transfer_action(unit, unit3, m, 1, unit))) {
+		prep_transfer_action(unit, unit3, m, 1, unit);
 		return TASK_PREPPED_ACTION;
 	    }
 	}
@@ -1079,7 +1086,7 @@ Task *task;
        for more. */
     tmpunit = unit;
     tmpm = m;
-    if (search_around(task->args[1], task->args[2], 3, extractable_test, &x1, &y1, 1)) {
+    if (search_around(x, y, 3, extractable_test, &x1, &y1, 1)) {
 	push_move_to_task(unit, x1, y1, 1);
 	task->args[1] = x1;  task->args[2] = y1;
 	return TASK_IS_INCOMPLETE;
@@ -1117,7 +1124,8 @@ Unit *unit;
 
     /* what about allies? */
     if (unit != tmpunit && unit_trusts_unit(unit, tmpunit)) {
-	if (unit->supply[tmpm] < um_storage_x(unit->type, tmpm)
+	if ((unit->supply[tmpm] < um_storage_x(unit->type, tmpm)
+	     || um_gives_to_treasury(unit->type, tmpm))
 	    /* this is a hack to prevent foragers from trying to deliver to each other */
 	    && !mobile(unit->type))
 	  return unit;
@@ -1897,6 +1905,13 @@ int x, y, dist;
 {
     Task *task = create_task(TASK_MOVE_TO);
 
+    /* Other (time-critical AI) code expects that our data is clean,
+       so filter it. */
+    if (!in_area(x, y)) {
+	run_warning("move-to task with bad dest %d,%d, replacing", x, y);
+	/* Replace with a location guaranteed to be valid. */
+	x = area.width / 2;  y = area.height / 2;
+    }
     task->args[0] = x;  task->args[1] = y;  task->args[2] = 0;
     task->args[3] = dist;
     task->args[4] = eitherway;
@@ -1993,14 +2008,14 @@ Unit *unit;
     add_task(unit, CLEAR_AGENDA, create_repair_task(unit));
 }
 
-/* This routine sets up a task to research a unit of the given type. */
+/* This routine sets up a task to develop a unit of the given type. */
 
 Task *
-create_research_task(unit, u2, n)
+create_develop_task(unit, u2, n)
 Unit *unit;
 int u2, n;
 {
-    Task *task = create_task(TASK_RESEARCH);
+    Task *task = create_task(TASK_DEVELOP);
 
     task->args[0] = u2;
     task->args[1] = n;
@@ -2008,11 +2023,11 @@ int u2, n;
 }
 
 void
-push_research_task(unit, u2, n)
+push_develop_task(unit, u2, n)
 Unit *unit;
 int u2, n;
 {
-    add_task(unit, 0, create_research_task(unit, u2, n));
+    add_task(unit, 0, create_develop_task(unit, u2, n));
 }
 
 Task *
