@@ -1,5 +1,5 @@
 /* Implementation of the transfer protocol for Xconq.
-   Copyright (C) 1996, 1997, 1998 Stanley T. Shebs.
+   Copyright (C) 1996-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -1164,15 +1164,15 @@ int u2, run;
 }
 
 void
-net_push_research_task(unit, u2, n)
+net_push_develop_task(unit, u2, n)
 Unit *unit;
 int u2, n;
 {
     if (my_rid == master_rid) {
-	push_research_task(unit, u2, n);
+	push_develop_task(unit, u2, n);
     }
     if (numremotes > 0) {
-	tmptask = create_research_task(unit, u2, n);
+	tmptask = create_develop_task(unit, u2, n);
 	broadcast_add_task(unit, 0, tmptask);
 	free_task(tmptask);
     }
@@ -1840,7 +1840,7 @@ char *buf;
 	    make_trial_assignments();
 	} else {
 	    /* Make sure we have space with which to concatenate. */
-	    if (strlen(downloadbuf) + strlen(buf) >= 100000) {
+	    if (strlen(downloadbuf) + strlen(buf) >= 200000) {
 		init_error("Exceeded download buffer size");
 		return;
 	    }
@@ -1859,7 +1859,7 @@ char *buf;
 	  case 'G':
 	    downloading = TRUE;
 	    if (downloadbuf == NULL)
-	      downloadbuf = xmalloc(100000);
+	      downloadbuf = xmalloc(200000);
 	    downloadbuf[0] = '\0';
 	    should_rebroadcast = FALSE;
 	    break;
