@@ -454,6 +454,7 @@ new_vp()
     vp->angle = 90;
     /* No vertical exaggeration by default. */
     vp->vertscale = 1;
+    vp->isodir = NORTHEAST;
     vp->cellwidth = area.cellwidth;
     /* If the cellwidth is not reasonable for drawing elevations, use
        an approximation based on the range of elevations and terrain
@@ -496,6 +497,49 @@ int x, y, *sxp, *syp;
 	      xnw -= area.width;
 
 	    *sxp = xnw * vp->hw + (y * vp->hw) / 2 - vp->sx;
+	} else if (vp->isometric) {
+	    switch (vp->isodir) {
+	      case NORTHEAST:
+		*sxp = x * vp->hch + (vp->hh - vp->hch);
+		*syp = ((area.height - 1 - y) * vp->hw) / 2;
+		*syp += ((area.width - 1 - x) * vp->hw) / 4;
+		break;
+	      case EAST:
+		*sxp = (area.height - 1 - y) * vp->hch + (vp->hh - vp->hch);
+		*syp = ((area.halfheight + area.width) * vp->hw) / 2;
+		*syp -= (x * vp->hw) / 2;
+		*syp -= (y * vp->hw) / 4;
+		break;
+	      case SOUTHEAST:
+		*sxp = (area.width - 1 - x + area.height - 1 - y) * vp->hch + (vp->hh - vp->hch);
+		*syp = ((area.width - 1 - x + y) * vp->hw) / 4;
+		break;
+	      case SOUTHWEST:
+		*sxp = (area.width - 1 - x) * vp->hch + (vp->hh - vp->hch);
+		*syp = (y * vp->hw) / 2 + (x * vp->hw) / 4;
+		break;
+	      case WEST:
+		*sxp = y * vp->hch + (vp->hh - vp->hch);
+		*syp = (x * vp->hw) / 2 + (y * vp->hw) / 4;
+		break;
+	      case NORTHWEST:
+		*sxp = (x + y) * vp->hch + (vp->hh - vp->hch);
+		*syp = ((area.height - 1 - y + x) * vp->hw) / 4;
+		break;
+	      default:
+		*sxp = *syp = 0;
+		break;
+	    }
+	    xw = wrapx(x);
+	    elev = (elevations_defined() ? elev_at(xw, y) : 0) - area.avgelev;
+	    /* We see the top of the terrain in the cell, not the bottom. */
+	    elev += t_thickness(terrain_at(xw, y));
+	    /* Exaggerate the vertical scale if requested. */
+	    elev *= vp->vertscale;
+	    offset = (elev * vp->hh) / vp->cellwidth;
+	    *syp -= offset;
+	    *sxp -= vp->sx;  *syp -= vp->sy;
+	    return;
 	} else {
 	    /* This calculation is currently appropriate for the Unix
                interface. */
@@ -563,7 +607,11 @@ int *sxp, *syp, *swp, *shp;
     if (unit->transport == NULL) {
 	xform_cell(vp, x, y, &sx, &sy);
 	/* Adjust to the unit box within the cell. */
-	sx += (vp->hw - vp->uw) / 2;  sy += (vp->hh - vp->uh) / 2;
+	sx += (vp->hw - vp->uw) / 2;
+	if (vp->isometric)
+	  sy += (vp->hw / 2 - vp->uh);
+	else
+	  sy += (vp->hh - vp->uh) / 2;
 	/* Figure out our position in this cell's stack. */
 	for_all_stack(x, y, unit2) {
 	    /* (should only count units visible to a given side) */
@@ -680,8 +728,20 @@ nearest_cell(vp, sx, sy, xp, yp, xfp, yfp)
 VP *vp;
 int sx, sy, *xp, *yp, *xfp, *yfp;
 {
-    int sxadj, sxfrac, syflipped, syfrac;
+    int sxadj, syadj, sxfrac, syflipped, syfrac;
 
+    if (vp->isometric) {
+	switch (vp->isodir) {
+	  case NORTHEAST:
+	    *xp = (sx + vp->sx) / vp->hch;
+	    syadj = sy + vp->sy - ((area.width - 1 - *xp) * vp->hw) / 4;
+	    *yp = area.height - 1 - (2 * syadj) / vp->hw;
+	    break;
+	  default:
+	    break;
+	}
+	return (in_area(*xp, *yp));
+    }
     sx -= vp->offsetx;  sy -= vp->offsety;
     /* Flip the raw y and then scale to hex coords. */
     syflipped = vp->totsh - (vp->sy + sy);
@@ -1892,6 +1952,8 @@ Side *side;
 Unit *unit, *other;
 int x, y;
 {
+    int z, m;
+
 #ifdef DESIGNERS
     /* Designers use this function to push units around, bound only by the
        limits on occupancy. */
@@ -1899,20 +1961,28 @@ int x, y;
 	return net_designer_teleport(unit, x, y, other);
     }
 #endif /* DESIGNERS */
+    z = unit->z;
     if (x != unit->x || y != unit->y) {
-	if (unit->act && unit->plan) { /* (should be more sophisticated test?) */
+	if (unit->act && unit->plan) { /* (should be more sophist test?) */
 	    if (distance(unit->x, unit->y, x, y) == 1) {
+		/* Destination is adjacent to us. */
+		/* First, handle the case where no unit was clicked on. */
 		if (other == NULL) {
 		    if (can_occupy_cell(unit, x, y)
-			&& valid(check_move_action(unit, unit, x, y, unit->z))) {
-			net_prep_move_action(unit, unit, x, y, unit->z);
+			&& valid(check_move_action(unit, unit, x, y, z))) {
+			net_prep_move_action(unit, unit, x, y, z);
+			return TRUE;
+		    } else if (can_extract_at(unit, x, y, &m)) {
+			net_set_collect_task(unit, m, x, y);
 			return TRUE;
 		    } else {
 			return FALSE;
 		    }
 		}
+		/* If a unit was clicked on, decide how to interact. */
 		if (unit_trusts_unit(unit, other)
-		    || (other->side == NULL && sides_allow_entry(unit, other))) {
+		    || (other->side == NULL
+			&& sides_allow_entry(unit, other))) {
 		    /* A friend, maybe get on it. */
 		    if (can_occupy(unit, other)) {
 			if (valid(check_enter_action(unit, unit, other))) {
@@ -1935,8 +2005,10 @@ int x, y;
 			}
 		    } else if (other->transport != NULL
 			       && can_occupy(unit, other->transport)) {
-			if (valid(check_enter_action(unit, unit, other->transport))) {
-			    net_prep_enter_action(unit, unit, other->transport);
+			if (valid(check_enter_action(unit, unit,
+						     other->transport))) {
+			    net_prep_enter_action(unit, unit,
+						  other->transport);
 			} else {
 			    /* (should schedule for next turn?) */
 			}
@@ -1949,36 +2021,57 @@ int x, y;
 			} else {
 			    /* (should schedule for next turn?) */
 			}
-		    } else if (valid(check_transfer_part_action(unit, unit, unit->hp, other))) {
-			net_prep_transfer_part_action(unit, unit, unit->hp, other);
+		    } else if (valid(check_transfer_part_action(unit, unit,
+								unit->hp, other))) {
+			net_prep_transfer_part_action(unit, unit,
+						      unit->hp, other);
+		    } else if (can_extract_at(unit, x, y, &m)
+			       || can_load_at(unit, x, y, &m)) {
+			/* (should test extraction from specific unit) */
+			net_set_collect_task(unit, m, x, y);
 		    } else if (can_occupy_cell(unit, x, y)
-			       && valid(check_move_action(unit, unit, x, y, unit->z))) {
-			net_prep_move_action(unit, unit, x, y, unit->z);
+			       && valid(check_move_action(unit, unit,
+							  x, y, z))) {
+			net_prep_move_action(unit, unit, x, y, z);
 		    } else {
 			return FALSE;
 		    }
 		} else {
-		    /* Somebody else's unit, try to victimize it in various ways,
-		       trying coexistence only as a last resort. */
+		    /* Somebody else's unit, try to victimize it in
+		       various ways, trying coexistence only as a last
+		       resort. */
 		    if (valid(check_capture_action(unit, unit, other))) {
 			net_prep_capture_action(unit, unit, other);
-		    } else if (valid(check_overrun_action(unit, unit, x, y, unit->z, 100))) {
-			net_prep_overrun_action(unit, unit, x, y, unit->z, 100);
-		    } else if (valid(check_attack_action(unit, unit, other, 100))) {
+		    } else if (valid(check_overrun_action(unit, unit, x, y,
+							  z, 100))) {
+			net_prep_overrun_action(unit, unit, x, y, z, 100);
+		    } else if (valid(check_attack_action(unit, unit, other,
+							 100))) {
 			net_prep_attack_action(unit, unit, other, 100);
-		    } else if (valid(check_fire_at_action(unit, unit, other, -1))) {
+		    } else if (valid(check_fire_at_action(unit, unit, other,
+							  -1))) {
 			net_prep_fire_at_action(unit, unit, other, -1);
-		    } else if (valid(check_detonate_action(unit, unit, x, y, unit->z))) {
-			net_prep_detonate_action(unit, unit, x, y, unit->z);
-		    } else if (valid(check_move_action(unit, unit, x, y, unit->z))) {
-			net_prep_move_action(unit, unit, x, y, unit->z);
+		    } else if (valid(check_detonate_action(unit, unit, x, y,
+							   z))) {
+			net_prep_detonate_action(unit, unit, x, y, z);
+		    } else if (can_extract_at(unit, x, y, &m)
+			       || can_load_at(unit, x, y, &m)) {
+			/* (should test extraction from specific unit) */
+			net_set_collect_task(unit, m, x, y);
+		    } else if (valid(check_move_action(unit, unit, x, y, z))) {
+			net_prep_move_action(unit, unit, x, y, z);
 		    } else {
 			return FALSE;
 		    }
 		}
 	    } else {
-		/* We're not adjacent to the destination, set up a move task. */
-		net_set_move_to_task(unit, x, y, 0);
+		/* We're not adjacent to the destination. */
+		if (can_extract_at(unit, x, y, &m)
+		    || can_load_at(unit, x, y, &m)) {
+		    net_set_collect_task(unit, m, x, y);
+		} else {
+		    net_set_move_to_task(unit, x, y, 0);
+		}
 	    }
 	} else {
 	    /* ??? can't act ??? */
@@ -2003,8 +2096,8 @@ int x, y;
 		    net_prep_attack_action(unit, unit, other, 100);
 		} else if (valid(check_fire_at_action(unit, unit, other, -1))) {
 		    net_prep_fire_at_action(unit, unit, other, -1);
-		} else if (valid(check_detonate_action(unit, unit, x, y, unit->z))) {
-		    net_prep_detonate_action(unit, unit, x, y, unit->z);
+		} else if (valid(check_detonate_action(unit, unit, x, y, z))) {
+		    net_prep_detonate_action(unit, unit, x, y, z);
 		} else {
 		    return FALSE;
 		}
@@ -2012,8 +2105,8 @@ int x, y;
 	} else if (unit->transport != NULL) {
 	    /* Unit is an occupant wanting to leave, but yet remain in
 	       the same cell as the transport. */
-	    if (valid(check_move_action(unit, unit, x, y, unit->z))) {
-		net_prep_move_action(unit, unit, x, y, unit->z);
+	    if (valid(check_move_action(unit, unit, x, y, z))) {
+		net_prep_move_action(unit, unit, x, y, z);
 	    } else {
 		return FALSE;
 	    }
