@@ -21,8 +21,10 @@ typedef struct a_vp {
     short hch;			/* vertical center-to-center distance */
     short uw, uh;		/* pixel dims of unit subcell */
     short angle;		/* Angle of perspective view */
-    short cellwidth;
+    int cellwidth;
     short vertscale;
+    short isometric;
+    short isodir;
     short wide_viewport;
     short center;		/* True if should center area in viewport */
     short offsetx, offsety;
