@@ -331,7 +331,7 @@ int flagacp;
 	  memset((char *) unit->act, 0, sizeof(ActorState));
 	/* Indicate that the action points have not been set. */
 	if (flagacp)
-	  unit->act->acp = u_acp_min(unit->type) - 1;
+	  unit->act->acp = unit->act->initacp = u_acp_min(unit->type) - 1;
 	/* Flag the action as undefined. */
 	unit->act->nextaction.type = ACTION_NONE;
     } else {
@@ -1210,7 +1210,8 @@ Unit *unit;
 		    if (terrain_view(side, x1, y1) == UNSEEN) {
 			set_terrain_view(side, x1, y1,
 					 buildtview(terrain_at(x1, y1)));
-			update_cell_display(side, x1, y1, TRUE);
+			update_cell_display(side, x1, y1,
+					    UPDATE_ALWAYS | UPDATE_ADJ);
 		    }
 		}
 	}
@@ -1348,6 +1349,9 @@ int reason;
 	cover_area(unit->side, unit, unit->transport, ux, uy, -1, -1);
 	/* Actually set the side slot of the unit here. */
 	set_unit_side(unit, newside);
+	/* Always redo the unit's number. */
+	unit->number = 0;
+	assign_unit_number(unit);
 	/* Cover it for the new side now. */
 	cover_area(unit->side, unit, unit->transport, -1, -1, ux, uy);
 	/* A freebie for the unit's previous side. */
@@ -1455,6 +1459,30 @@ Side *side;
     return side->uavail[u];
 }
 
+int
+type_ever_available(u, side)
+int u;
+Side *side;
+{
+    int u2;
+    Unit *unit;
+
+    if (!type_allowed_on_side(u, side))
+      return FALSE;
+
+    for_all_side_units(side, unit) {
+	if (unit->type == u)
+	  return TRUE;
+    }
+    for_all_unit_types(u2) {
+	if (uu_acp_to_create(u2, u) > 0
+	    && type_allowed_on_side(u2, side))
+	  return TRUE;
+    }
+    /* (should add tests for capture etc) */
+    return FALSE;
+}
+
 /* TRUE if side has advance to build this unit. */
 
 int
@@ -2620,7 +2648,7 @@ int u, s, x, y;
 	/* (should undo creation of unit?) */
 	return NULL;
     }
-    update_cell_display(side, x, y, TRUE);
+    update_cell_display(side, x, y, UPDATE_ALWAYS);
     update_unit_display(side, newunit, TRUE);
     return newunit;
 }
