@@ -64,9 +64,9 @@ static char *padbuf;
 
 /* The random state *must* be at least 32 bits. */
 
-long initrandstate = -1;
+int initrandstate = -1;
 
-long randstate = -1;
+int randstate = -1;
 
 /* Seed can come from elsewhere, for repeatability.  Otherwise, it comes
    from the current time, scaled down to where 32-bit arithmetic won't
@@ -88,11 +88,11 @@ int seed;
 	randstate = seed;
     } else {
     	time(&tm);
-    	randstate = (long) tm;
+    	randstate = (int) tm;
     }
     /* Whatever its source, put the randstate into known range (0 - 99999). */
     randstate = ABS(randstate);
-    randstate %= 100000L;
+    randstate %= 100000;
     /* This is kept around for the sake of error reporting. */
     initrandstate = randstate;
 }
@@ -104,8 +104,20 @@ int
 xrandom(m)
 int m;
 {
-    randstate = (8121 * randstate + 28411) % 134456L;
-    return ((m * randstate) / 134456L);
+    int rslt;
+    long long bignum;
+
+    randstate = (8121 * randstate + 28411) % 134456;
+    /* Random numbers in too large of a range will overflow 32-bit
+       arithmetic, but Xconq rarely needs the large ranges, so only do
+       the 64-bit arithmetic if necessary. */
+    if (m <= 10000) {
+	rslt = (m * randstate) / 134456;
+    } else {
+	bignum = m * ((long long) randstate);
+	rslt = bignum / 134456;
+    }
+    return rslt;
 }
 
 /* Percentage probability, with bounds checking. */
