@@ -294,6 +294,8 @@ final_init_world()
 	    sum += rowavg;
 	}
 	area.avgelev = sum / count;
+	Dprintf("Area elevations range %d to %d, averaging %d\n",
+		area.minelev, area.maxelev, area.avgelev);
     }
     /* Calculate extent and position of geographical features. */
     compute_all_feature_centroids();
@@ -742,7 +744,7 @@ int x, y, t2;
 	      update = TRUE;
 	}
 	if (update) {
-	    update_cell_display(side, x, y, TRUE);
+	    update_cell_display(side, x, y, UPDATE_ALWAYS | UPDATE_ADJ);
 	}
     }
     /* There might be catastrophic consequences for any units here. */
@@ -2071,7 +2073,7 @@ int x, y;
     /* Only do anything if we're actually changing to a different type. */
     if (terrain_at(x, y) != tmpttype) {
 	set_terrain_at(x, y, tmpttype);
-	update_cell_display(tmpside, x, y, FALSE);
+	update_cell_display(tmpside, x, y, UPDATE_ALWAYS | UPDATE_ADJ);
     }
 }
 
@@ -2090,9 +2092,7 @@ int x, y, dir, t, mode;
     oldbord = border_at(x, y, dir, t);
     set_border_at(x, y, dir, t, (mode < 0 ? !oldbord : mode));
     if (oldbord != border_at(x, y, dir, t)) {
-	update_cell_display(side, x, y, TRUE);
-	if (point_in_dir(x, y, dir, &x1, &y1))
-	  update_cell_display(side, x1, y1, TRUE);
+	update_cell_display(side, x, y, UPDATE_ALWAYS | UPDATE_ADJ);
     }
 }
 
@@ -2109,9 +2109,7 @@ int x, y, dir, t, mode;
     oldconn = connection_at(x, y, dir, t);
     set_connection_at(x, y, dir, t, (mode < 0 ? !oldconn : mode));
     if (oldconn != connection_at(x, y, dir, t)) {
-	update_cell_display(side, x, y, TRUE);
-	if (point_in_dir(x, y, dir, &x1, &y1))
-	  update_cell_display(side, x1, y1, TRUE);
+	update_cell_display(side, x, y, UPDATE_ALWAYS | UPDATE_ADJ);
     }
 }
 
@@ -2137,7 +2135,7 @@ int x, y;
 
     if (olddepth != tmpint) {
 	set_aux_terrain_at(x, y, tmpttype, tmpint);
-	update_cell_display(tmpside, x, y, TRUE);
+	update_cell_display(tmpside, x, y, UPDATE_ALWAYS | UPDATE_ADJ);
     }
 }
 
@@ -2163,11 +2161,7 @@ int x, y;
     if (oldpeop != tmpint) {
 	set_people_side_at(x, y, tmpint);
 	/* Appearance of adjacent cells may change also. */
-	for_all_directions(dir) {
-	    if (point_in_dir(x, y, dir, &x1, &y1))
-	      update_cell_display(tmpside, x1, y1, TRUE);
-	}
-	update_cell_display(tmpside, x, y, TRUE);
+	update_cell_display(tmpside, x, y, UPDATE_ALWAYS | UPDATE_ADJ);
 
     }
 }
@@ -2194,11 +2188,7 @@ int x, y;
     if (oldpeop != tmpint) {
 	set_control_side_at(x, y, tmpint);
 	/* Appearance of adjacent cells may change also. */
-	for_all_directions(dir) {
-	    if (point_in_dir(x, y, dir, &x1, &y1))
-	      update_cell_display(tmpside, x1, y1, TRUE);
-	}
-	update_cell_display(tmpside, x, y, TRUE);
+	update_cell_display(tmpside, x, y, UPDATE_ALWAYS | UPDATE_ADJ);
     }
 }
 
@@ -2240,11 +2230,7 @@ int x, y;
 	}
 	/* Might need to redraw feature borders in adjacent cells. */
 	/* (should test before redirecting redraw?) */
-	for_all_directions(dir) {
-	    if (point_in_dir(x, y, dir, &x1, &y1))
-	      update_cell_display(tmpside, x1, y1, TRUE);
-	}
-	update_cell_display(tmpside, x, y, TRUE);
+	update_cell_display(tmpside, x, y, UPDATE_ALWAYS | UPDATE_ADJ);
     }
 }
 
@@ -2276,12 +2262,7 @@ int x, y;
 	if (n > area.maxelev)
 	  area.maxelev = n;
 	/* (should force recomputation of contour intervals) */
-	for_all_directions(dir) {
-	    if (point_in_dir(x, y, dir, &x1, &y1)) {
-		update_cell_display(tmpside, x1, y1, TRUE);
-	    }
-	}
-	update_cell_display(tmpside, x, y, TRUE);
+	update_cell_display(tmpside, x, y, UPDATE_ALWAYS | UPDATE_ADJ);
     }
 }
 
@@ -2305,7 +2286,7 @@ int x, y;
     n = max(t_temp_min(t), min(tmpint, t_temp_max(t)));
     if (n != oldtemp) {
 	set_temperature_at(x, y, n);
-	update_cell_display(tmpside, x, y, TRUE);
+	update_cell_display(tmpside, x, y, UPDATE_ALWAYS);
     }
 }
 
@@ -2331,7 +2312,7 @@ int x, y;
 
     if (oldm != tmpint) {
 	set_material_at(x, y, tmpmtype, tmpint);
-	update_cell_display(tmpside, x, y, TRUE);
+	update_cell_display(tmpside, x, y, UPDATE_ALWAYS);
     }
 }
 
@@ -2375,7 +2356,7 @@ int x, y;
 	changed = TRUE;
     }
     if (changed)
-         update_cell_display(tmpside, x, y, TRUE);
+         update_cell_display(tmpside, x, y, UPDATE_ALWAYS | UPDATE_ADJ);
 }
 
 /* Winds painting. */
@@ -2399,7 +2380,7 @@ int x, y;
 
     if (oldw != tmpint) {
 	set_raw_wind_at(x, y, tmpint);
-	update_cell_display(tmpside, x, y, TRUE);
+	update_cell_display(tmpside, x, y, UPDATE_ALWAYS);
     }
 }
 
