@@ -1,5 +1,5 @@
 /* Xconq game module writing.
-   Copyright (C) 1987-1989, 1991-1998 Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -280,14 +280,6 @@ Module *module;
 	  write_tables(module->compress_tables);
 	if (module->def_all || module->def_globals)
 	  write_globals();
-	/* The synthesis method list is special. */
-	if (!downloading) {
-	    start_form(key(K_SET));
-	    add_to_form("synthesis-methods");
-	    add_to_form("nil");
-	    end_form();
-	    newline_form();
-	}
 	if (module->def_all || module->def_scoring)
 	  write_scorekeepers();
 	doreshape = reshape_the_output(module);
@@ -1320,12 +1312,11 @@ write_globals()
 	    }
 	} else if (vardefns[i].objgetter != NULL) {
 	    val = (*(vardefns[i].objgetter))();
-	    /* (should recognize the function-defaulted Lisp values,
-	       but since the default-generating functions may allocate,
-	       this could only work if the original values were recorded;
-	       only synth list and side library involved, so just write
-	       them always) */
-	    if (complete || val != lispnil) {
+	    if (complete
+		|| val != lispnil
+		/* Must write out explicit nil setting if the default
+		   function is defined, so it won't re-run on startup. */
+		|| (val == lispnil && vardefns[i].dfltfn != NULL)) {
 		start_form(key(K_SET));
 		add_to_form(vardefns[i].name);
 		space_form();
@@ -1455,7 +1446,10 @@ Side *side;
     write_lisp_prop(key(K_UNITS), side->possible_units, lispnil, FALSE, FALSE, TRUE);
     write_str_prop(key(K_CLASS), side->sideclass, "", FALSE, TRUE);
     write_bool_prop(key(K_ACTIVE), side->ingame, 1, FALSE, TRUE);
-    write_bool_prop(key(K_EVER_ACTIVE), side->everingame, 1, FALSE, TRUE);
+    /* side->everingame is only interesting if it differs from ingame,
+       so use the value of ingame as the default. */
+    write_bool_prop(key(K_EVER_ACTIVE), side->everingame, side->ingame,
+		    FALSE, TRUE);
     write_num_prop(key(K_PRIORITY), side->priority, 0, FALSE, TRUE);
     write_num_prop(key(K_STATUS), side->status, 0, FALSE, TRUE);
     if (!debugging_state_sync)
@@ -2069,7 +2063,9 @@ Unit *unit;
     write_num_prop(key(K_TRK), unit->tracking, NOSIDES, FALSE, FALSE);
     write_utype_value_list(key(K_TP), unit->tooling, 0, FALSE);
     write_side_value_list(key(K_OPINIONS), unit->opinions, 0, FALSE);
-    write_mtype_value_list(key(K_M), unit->supply, 0, FALSE);
+    /* Getting the defaults right for the supply list requires more
+       cleverness, alway write for now. */
+    write_mtype_value_list(key(K_M), unit->supply, -1, FALSE);
     write_num_prop(key(K_POINT_VALUE), unit_point_value(unit), -1, FALSE, FALSE);
     write_num_prop(key(K_APPEAR), unit_appear_turn(unit), -1, FALSE, FALSE);
     /* (should do appear x,y also here?) */
@@ -2079,7 +2075,7 @@ Unit *unit;
 // ---ADVANCED UNIT SUPPORT -------------------------------------------------------
 
 	if (u_advanced(unit->type)) {
-		write_num_prop(key(K_SIZE), unit->size, 1, FALSE, FALSE);		
+		write_num_prop(key(K_SIZE), unit->size, 0, FALSE, FALSE);		
 		write_num_prop(key(K_REACH), unit->reach, 2, FALSE, FALSE);		
 		write_num_prop(key(K_USEDCELLS), unit->usedcells, 0, FALSE, FALSE);		
 		write_num_prop(key(K_MAXCELLS), unit->maxcells, 1, FALSE, FALSE);		
@@ -2115,12 +2111,9 @@ Unit *unit;
 	   newline_form();
 	   space_form();
 	}
-	if (act->acp != acp)
-	  write_num_prop(key(K_ACP), act->acp, acp, FALSE, FALSE);
-	if (act->initacp != acp)
-	  write_num_prop(key(K_ACP0), act->initacp, acp, FALSE, FALSE);
-	if (act->actualmoves)
-	  write_num_prop(key(K_AM), act->actualmoves, 0, FALSE, FALSE);
+	write_num_prop(key(K_ACP), act->acp, acp, FALSE, FALSE);
+	write_num_prop(key(K_ACP0), act->initacp, acp, FALSE, FALSE);
+	write_num_prop(key(K_AM), act->actualmoves, 0, FALSE, FALSE);
 	if (act->nextaction.type != ACTION_NONE) {
 	    space_form();
 	    start_form(key(K_A));
