@@ -1679,7 +1679,7 @@ draw_unit_info(Map *map)
 						infobuf[0] = '\0';
 						row++;
 					}
-					task_desc(infobuf+strlen(infobuf), unit->side, task);
+					task_desc(infobuf+strlen(infobuf), unit->side, unit, task);
 					if (task->next != NULL)
 					  strcat(infobuf, "; ");
 					++n;
