@@ -2392,7 +2392,7 @@ enable_commands_for_unit(MenuHandle menu, MenuHandle menu2, Unit *unit)
 		&& (unit->transport == NULL
 			|| !completed(unit->transport)
 			|| uu_occ_can_build(unit->transport->type, u))
-		|| can_research(unit)) {
+		|| can_develop(unit)) {
 		EnableItem(menu, miPlayBuild);
 	}
 	if (can_repair(unit)) {
