@@ -1,5 +1,5 @@
 /* Handling of assorted minor windows for the Mac interface to Xconq.
-   Copyright (C) 1992-1998 Stanley T. Shebs.
+   Copyright (C) 1992-1999 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -76,7 +76,7 @@ static int maxtypewid;
 static int constructmargin = 5;
 static int constructtop = 32;
 
-/* This is the vector of units that can do construction or research. */
+/* This is the vector of units that can do construction or development. */
 
 static UnitVector *constructor_units = NULL;
 
@@ -89,7 +89,7 @@ int editedrunlength = -1;
 
 /* (this is referenced from the end-turn command, so can't make static) */
 ControlHandle constructbutton;
-static ControlHandle researchbutton;
+static ControlHandle developbutton;
 
 TEHandle run_length_text = nil;
 
@@ -568,7 +568,7 @@ create_construction_window()
 		constructionwin = GetNewWindow(wConstruction, NULL, (WindowPtr) -1L);
 	}
 	constructbutton = GetNewControl(cConstructButton, constructionwin);
-	researchbutton = GetNewControl(cResearchButton, constructionwin);
+	developbutton = GetNewControl(cResearchButton, constructionwin);
 	SetPort(constructionwin);
 	TextFont(small_font_id);
 	TextSize(small_font_size);
@@ -825,12 +825,12 @@ do_mouse_down_construction(Point mouse, int mods)
 				return;
 			}
 		}
-	} else if (control == researchbutton) {
+	} else if (control == developbutton) {
 		unit = get_selected_construction_unit();
 		if (unit != NULL) {
 			u = get_selected_construction_type();
 			if (u != NONUTYPE) {
-				net_push_research_task(unit, u, u_tech_to_build(u));
+				net_push_develop_task(unit, u, u_tech_to_build(u));
 				update_construction_unit_list(unit);
 				if (modal_construction && window_behind_construction != nil) {
 					SelectWindow(window_behind_construction);
@@ -1040,7 +1040,7 @@ update_type_list_for_unit(Unit *unit)
 void
 adjust_construction_controls()
 {
-	int u, canconstruct = FALSE, canresearch = FALSE, len;
+	int u, canconstruct = FALSE, candevelop = FALSE, len;
 	Unit *unit;
 
 	unit = get_selected_construction_unit();
@@ -1049,12 +1049,12 @@ adjust_construction_controls()
 		if (u != NONUTYPE) {
 			if (uu_acp_to_create(unit->type, u) > 0)
 			  canconstruct = TRUE;
-			if (uu_acp_to_research(unit->type, u) > 0)
-			  canresearch = TRUE;
+			if (uu_acp_to_develop(unit->type, u) > 0)
+			  candevelop = TRUE;
 		}
 	}
 	HiliteControl(constructbutton, (canconstruct ? 0 : 255));
-	HiliteControl(researchbutton, (canresearch ? 0 : 255));
+	HiliteControl(developbutton, (candevelop ? 0 : 255));
 	draw_construction_default();
 	/* If there is doctrine on construction run length, use it to seed the
 	   run length in the dialog. */
@@ -1475,7 +1475,7 @@ draw_unit_closeup(UnitCloseup *unitcloseup)
 			DrawText(infobuf, 0, strlen(infobuf));
     	}
     	for_all_tasks(plan, task) {
-			task_desc(infobuf, unit->side, task);
+			task_desc(infobuf, unit->side, unit, task);
 			sy += closeup_spacing;			
 			MoveTo(sx, sy);
 			DrawText(infobuf, 0, strlen(infobuf));
