@@ -869,6 +869,11 @@ do_map(Side *side)
     } else if (strcmp(cmdargstr, "grid") == 0) {
 	vp->draw_grid = !vp->draw_grid;
 	redraw_map(map);
+    } else if (strcmp(cmdargstr, "iso") == 0) {
+	vp->isometric = TRUE;
+	if (map->prefixarg >= 0)
+	  vp->vertscale = map->prefixarg;
+	redraw_map(map);
     } else if (strcmp(cmdargstr, "lighting") == 0) {
 	vp->draw_lighting = !vp->draw_lighting;
 	redraw_map(map);
@@ -877,6 +882,11 @@ do_map(Side *side)
 	redraw_map(map);
     } else if (strcmp(cmdargstr, "meridians") == 0) {
 	vp->draw_latlong = !vp->draw_latlong;
+	if (map->prefixarg > 0)
+	  vp->latlong_interval = map->prefixarg;
+	redraw_map(map);
+    } else if (strcmp(cmdargstr, "over") == 0) {
+	vp->isometric = FALSE;
 	redraw_map(map);
     } else if (strcmp(cmdargstr, "people") == 0) {
 	vp->draw_people = !vp->draw_people;
@@ -902,6 +912,14 @@ do_map(Side *side)
     } else if (strcmp(cmdargstr, "polygons") == 0) {
 	toggle_mapw_polygons(map);
 	redraw_map(map);
+    } else if (strcmp(cmdargstr, "rotl") == 0) {
+	vp->isodir = left_dir(vp->isodir);
+	printf("Looking %s\n", dirnames[vp->isodir]);
+	redraw_map(map);
+    } else if (strcmp(cmdargstr, "rotr") == 0) {
+	vp->isodir = right_dir(vp->isodir);
+	printf("Looking %s\n", dirnames[vp->isodir]);
+	redraw_map(map);
     } else if (strcmp(cmdargstr, "seeall") == 0) {
 	if (!side->may_set_see_all) {
 	    cmd_error(side, "May not set to see all!");
@@ -1638,7 +1656,9 @@ int cancel;
 	    if (in_play(leader)
 		&& leader != unit
 		&& trusted_side(unit->side, leader->side)) {
-		net_set_formation(unit, leader, leader->x - unit->x, leader->y - unit->y, 1, 1);
+		net_set_formation(unit, leader,
+				  unit->x - leader->x, unit->y - leader->y,
+				  1, 1);
 	    } else {
 		cmd_error(side, "Nobody here to form on!");
 	    }
@@ -1652,7 +1672,13 @@ int cancel;
 void
 do_set_view_angle(Side *side)
 {
-    cmd_error(side, "Not implemented.");
+    Map *map = dside->ui->curmap;
+    VP *vp = widget_vp(map);
+
+    vp->isometric = !vp->isometric;
+    if (map->prefixarg >= 0)
+      vp->vertscale = map->prefixarg;
+    redraw_map(map);
 }
 
 void
