@@ -120,6 +120,7 @@ typedef struct a_ui {
     XColor *unit_name_color;
     XColor *cellcolor[MAXTTYPES];	/* the color of each terrain type */
     XColor *cellfgcolor[MAXTTYPES]; /* the "fg" color of each terrain type */
+    XColor *cellshades[MAXTTYPES][5];
     XColor *colors[MAXSIDES][3];
     int numcolors[MAXSIDES];
     XColor **unitcolors;
@@ -175,11 +176,10 @@ typedef struct a_ui {
     char terrchars[NUMPOWERS][MAXTTYPES];
 
     Pixmap hexpics[NUMPOWERS], bhexpics[NUMPOWERS];
-    Pixmap hexchars[NUMPOWERS], bhexchars[NUMPOWERS];
+    Pixmap hexisopics[NUMPOWERS];
     /* Unit drawing machinery */
     Pixmap unitpics[NUMPOWERS][MAXUTYPES];
     Pixmap unitmasks[NUMPOWERS][MAXUTYPES];
-    char unitchars[NUMPOWERS][MAXUTYPES];
 
     /* Emblem drawing machinery */
     Pixmap emblempics[MAXSIDES];
