@@ -11,11 +11,26 @@
 wm title . "Xconq Main Window"
 wm withdraw .
 
+# Asking for a font by size causes problems for some X servers.
+
+set textfont fixed
+#set textfont {-size 12}
+puts stdout "text font is [ font actual $textfont ]"
+set boldfont fixed
+#set boldfont {-size 12 -weight bold}
+puts stdout "bold font is [ font actual $boldfont ]"
+
+# Asking for a specific color sometimes loses...
+
+set progress_bar_color black
+set progress_bar_color "#010101"
+
 set lineheight 5
 
+set list_icon_size 16
 set list_icon_size 32
 
-set dblbuffer 1
+set dblbuffer 0
 
 set designer 0
 
@@ -28,12 +43,15 @@ set handling_key 0
 
 set curunit 0
 
-set debug 1
+set debug 0
 
 if { $debug == 1 } {
     set dblbuffer 0
 }
 
+set isometric_state normal
+set isometric_state disabled
+
 # The initial splash screen.  This proc also sets up the framing and
 # buttons that later screens will use.
 
@@ -397,7 +415,7 @@ proc world_size_ok_cmd {} {
     set base .newgame.top.sizes
     set new_width [ $base.fwidth get ]
     set new_height [ $base.fheight get ]
-    set new_circumference [ $base.f get ]
+    set new_circumference [ $base.fcircumf get ]
     set new_latitude [ $base.flat get ]
     set new_longitude [ $base.flon get ]
     # Use the cancellation steps to finish here.
@@ -700,21 +718,18 @@ proc do_initial_setup {} {
 # individual units, lists of sides, unit types, etc.
 
 proc create_map_window {} {
+    global textfont boldfont
     global lineheight
     global list_icon_size
     global dblbuffer
     global dside
+    global isometric_state
 
-    set textfont fixed
-#{-size 12}
-    set boldfont fixed
-#{-size 12 -weight bold}
     set asc [ font metrics $textfont -ascent ]
     set dsc [ font metrics $textfont -descent ]
     set lineheight [ expr $asc + $dsc + 1 ]
 
     set nums [ numsides ]
-    set numu [ numutypes ]
 
     set mapn 1
     set map .m$mapn
@@ -781,6 +796,18 @@ proc create_map_window {} {
 	    -width 24 -height 24 \
 	    -command { execute_long_command "zoom-out" }
     pack $map.leftside.buttons.zoomout -side top
+    button $map.leftside.buttons.iso -bitmap iso -state $isometric_state \
+	    -width 24 -height 24 \
+	    -command { execute_long_command "map iso" }
+    pack $map.leftside.buttons.iso -side top
+    button $map.leftside.buttons.rotl -bitmap rotl -state $isometric_state \
+	    -width 24 -height 24 \
+	    -command { execute_long_command "map rotl" }
+    pack $map.leftside.buttons.rotl -side top
+    button $map.leftside.buttons.rotr -bitmap rotr -state $isometric_state \
+	    -width 24 -height 24 \
+	    -command { execute_long_command "map rotr" }
+    pack $map.leftside.buttons.rotr -side top
 
     text $map.leftside.mouseover -height 1 -font $textfont
     pack $map.leftside.mouseover -side top -fill x
@@ -816,7 +843,7 @@ proc create_map_window {} {
     frame $map.rightside.turnf -borderwidth 2 -relief sunken
     pack $map.rightside.turnf -side top -fill x -expand true
 
-    canvas $map.rightside.turnf.turn -height 20
+    canvas $map.rightside.turnf.turn -height [ expr $lineheight + 4 ]
     pack $map.rightside.turnf.turn -side top -fill x -expand true
 
     frame $map.rightside.gamef -borderwidth 2 -relief sunken
@@ -824,6 +851,8 @@ proc create_map_window {} {
 
     set game_entry_height [ expr 2 * $lineheight + 20 ]
     set game_win_height [ expr $nums * $game_entry_height ]
+    set numtreas [ numtreasury ]
+    incr game_win_height [ expr (($numtreas + 1) / 2) * $lineheight ]
     set actualheight $game_win_height
     set limitheight [ expr $mainhgt / 2 ]
     if { $actualheight > $limitheight } {
@@ -846,6 +875,7 @@ proc create_map_window {} {
     frame $map.rightside.listf -borderwidth 2 -relief sunken
     pack $map.rightside.listf -side top -expand true -fill x
 
+    set numu [ numutypes_available $dside ]
     set list_entry_height [ expr $list_icon_size + 6 ]
     set listwinheight [ expr $lineheight + $numu * $list_entry_height ]
     canvas $map.rightside.listf.unitlist -height $listwinheight \
@@ -892,11 +922,28 @@ proc create_map_window {} {
     $unitinfo create text $col2 $ypos -tag s3 -anchor sw -font $textfont
 
     set turnpane $map.rightside.turnf.turn
-    $turnpane create text 24 [ expr $lineheight + 2 ] -tag the_date -anchor sw -font $textfont
+    $turnpane create text 4 4 -tag the_date -anchor nw -font $boldfont
 
-    set gamepane $map.rightside.gamef.game
+    fill_in_side_list $map
 
-    set bgcolor [ $gamepane cget -background ]
+    fill_in_unit_type_list $map
+
+    make_normal_bindings $map
+}
+
+proc fill_in_side_list { map } {
+    global textfont boldfont
+    global lineheight
+    global progress_bar_color
+    global dside
+
+    set nums [ numsides ]
+    set dside [ dside ]
+    set numtreas [ numtreasury ]
+
+    set sidelist $map.rightside.gamef.game
+    set game_entry_height [ expr 2 * $lineheight + 20 ]
+    set bgcolor [ $sidelist cget -background ]
 
     set sy 0
     for { set i 1 } { $i <= $nums } { incr i } {
@@ -906,44 +953,73 @@ proc create_map_window {} {
 	set rbot [ expr $sy + $lineheight + 12 ]
 	set scy [ expr $sy + (2 * $lineheight) + 15 ]
 	if { $i > 1 } {
-	    $gamepane create line 0 $sy 400 $sy -fill gray
+	    $sidelist create line 0 $sy 400 $sy -fill gray
 	}
-	imfsample $gamepane.e$i -width 16 -height 16 -bg $bgcolor
-	$gamepane.e$i add imf [ side_emblem $i ]
-	$gamepane create window 4 [ expr $sy + 4 ] -window $gamepane.e$i \
+	imfsample $sidelist.e$i -width 16 -height 16 -bg $bgcolor
+	$sidelist.e$i add imf [ side_emblem $i ]
+	$sidelist create window 4 [ expr $sy + 4 ] -window $sidelist.e$i \
 		-anchor nw
-	$gamepane create text 24 $tsy -tag s$i -anchor sw -font $textfont
+	$sidelist create text 24 $tsy -tag s$i -anchor sw -font $textfont
 	# (should make conditional on liveness of side)
-	$gamepane create rect 23 $rtop 125 $rbot -tag frame$i
-	$gamepane create rect 24 [ expr $rtop + 1 ] 24 $rbot -tag left$i \
-		-fill "dark blue" -outline ""
-	$gamepane create rect 24 [ expr $rtop + 1 ] 24 $rbot -tag resv$i \
+	$sidelist create rect 23 $rtop 125 $rbot -tag frame$i
+	$sidelist create rect 24 [ expr $rtop + 1 ] 24 $rbot -tag left$i \
+		-fill $progress_bar_color -outline ""
+	$sidelist create rect 24 [ expr $rtop + 1 ] 24 $rbot -tag resv$i \
 		-fill gray -outline ""
-	$gamepane create text 24 $scy -tag score0_$i -text sc0_$i -anchor sw -font $textfont
+	$sidelist create text 24 $scy -tag score0_$i -text sc0_$i -anchor sw -font $textfont
 	# Start the loss line and victory laurels offscreen. 
-	$gamepane create line 400 $msy 1600 $msy -tag lost$i -fill black
-	$gamepane create bitmap 400 [ expr $sy + 22 ] -bitmap laurels -tag won$i -anchor nw
+	$sidelist create line 400 $msy 1600 $msy -tag lost$i -fill black
+	$sidelist create bitmap 400 [ expr $sy + 22 ] -bitmap laurels -tag won$i -anchor nw
 	incr sy $game_entry_height
+	# Possibly add treasury materials.
+	if { $numtreas > 0 && $i == $dside } {
+	    # Make two columns.
+	    set rows [ expr ($numtreas + 1) / 2 ]
+	    set j 0
+	    for { set row 0 } { $row < $rows } { incr row } {
+		$sidelist create text 40 $sy -text m$i,$j -tag m$i,$j -anchor ne -font $textfont
+		$sidelist create text 45 $sy -text [ mtype_name [ mtype_actual $j ] ] -anchor nw -font $textfont
+		incr j
+		$sidelist create text 120 $sy -text m$i,$j -tag m$i,$j -anchor ne -font $textfont
+		$sidelist create text 125 $sy -text [ mtype_name [ mtype_actual $j ] ] -anchor nw -font $textfont
+		incr j
+		incr sy $lineheight
+	    }
+	}
     }
 
-    set dside [ dside ]
+    $sidelist itemconfigure s$dside -font $boldfont
+}
 
-    $gamepane itemconfigure s$dside -font $boldfont
+proc fill_in_unit_type_list { map } {
+    global textfont boldfont
+    global lineheight
+    global list_icon_size
+    global dside
 
     set unitlist $map.rightside.listf.unitlist
+
+    set bgcolor [ $unitlist cget -background ]
+
     $unitlist create text [ expr $list_icon_size + 16 ] $lineheight -text "Type"\
 	    -anchor se -font $textfont
     $unitlist create text [ expr $list_icon_size + 44 ] $lineheight -text "Num" \
 	    -anchor s -font $textfont
 
     set sy 16
+    set list_entry_height [ expr $list_icon_size + 4 ]
+    set numu [ numutypes_available $dside ]
     for { set i 0 } { $i < $numu } { incr i } {
-	# (should filter out or diminish entries for unavailable types)
+	# (should filter out entries for unavailable types)
 	# Compute the y position of text items.
 	set tsy [ expr $sy + $list_icon_size / 2 + $lineheight / 2 ]
 	imfsample $unitlist.u$i -width $list_icon_size -height $list_icon_size \
 		-bg $bgcolor
-	$unitlist.u$i add imf [ u_image_name $i ]
+	$unitlist.u$i add imf [ u_image_name [ utype_actual $i ] ]
+	# Add the side emblem as a second image, but don't display it
+	# as an image; instead declare as the "emblem".
+	$unitlist.u$i add imf [ side_emblem $dside ]
+	$unitlist.u$i emblem 1
 	$unitlist create window 4 $sy -window $unitlist.u$i -anchor nw
 	$unitlist create text [ expr $list_icon_size + 16 ] $tsy -tag u$i \
 		-anchor s -font $textfont
@@ -951,15 +1027,23 @@ proc create_map_window {} {
 		-anchor se -font $textfont
 	$unitlist create text [ expr $list_icon_size + 44 ] $tsy -tag i$i \
 		-anchor sw -font $textfont
+	$unitlist create text [ expr $list_icon_size + 80 ] $tsy -tag name$i \
+		-anchor sw -font $textfont
 	set rtop [ expr $sy - 1 ]
 	set rbot [ expr $sy + $list_icon_size + 1 ]
 	$unitlist create rect 3 $rtop [ expr $list_icon_size + 5 ] $rbot -tag rect$i -outline $bgcolor
 	incr sy $list_entry_height
     }
-
-    make_normal_bindings $map
 }
 
+set can_act 0
+set can_plan 0
+set can_move 0
+set can_return 0
+set can_build 0
+set can_attack 0
+set can_fire 0
+
 set can_see_people 0
 set can_see_control 0
 set can_see_elev 0
@@ -1058,7 +1142,7 @@ proc create_map_menus { map } {
     $map.menubar.play add separator
     $map.menubar.play add command -label "Wake" -accelerator "w" \
 	    -command { execute_long_command "wake" }
-    $map.menubar.play add command -label "Wake Occupants" -accelerator "W" \
+    $map.menubar.play add command -label "Wake All" -accelerator "W" \
 	    -command { execute_long_command "wakeall" }
     $map.menubar.play add command -label "Sleep" -accelerator "s" \
 	    -command { execute_long_command "sleep" }
@@ -1257,27 +1341,29 @@ proc create_map_menus { map } {
 
 proc adjust_play_menu { map } {
     global curunit
+    global can_act can_plan can_move can_return can_build can_attack can_fire
 
-    adjust_menu_entry $map play "Move To" $curunit
-    adjust_menu_entry $map play "Return" $curunit
-    adjust_menu_entry $map play "Set Formation" $curunit
-    adjust_menu_entry $map play "Wake" $curunit
-    adjust_menu_entry $map play "Wake Occupants" $curunit
-    adjust_menu_entry $map play "Sleep" $curunit
-    adjust_menu_entry $map play "Reserve" $curunit
-    adjust_menu_entry $map play "Idle" $curunit
-    adjust_menu_entry $map play "Delay" $curunit
-    adjust_menu_entry $map play "Build" $curunit
+    adjust_menu_entry $map play "Move To" [ expr $curunit && $can_move ]
+    adjust_menu_entry $map play "Return" [ expr $curunit && $can_return ]
+    adjust_menu_entry $map play "Set Formation" [ expr $curunit && $can_move ]
+    adjust_menu_entry $map play "Wake" [ expr $curunit && $can_plan ]
+    adjust_menu_entry $map play "Wake All" [ expr $curunit && $can_plan ]
+    adjust_menu_entry $map play "Sleep" [ expr $curunit && $can_plan ]
+    adjust_menu_entry $map play "Reserve" [ expr $curunit && $can_plan ]
+    adjust_menu_entry $map play "Idle" [ expr $curunit && $can_plan ]
+    adjust_menu_entry $map play "Delay" [ expr $curunit && $can_plan ]
+    adjust_menu_entry $map play "Build" [ expr $curunit && $can_build ]
     adjust_menu_entry $map play "Repair" $curunit
-    adjust_menu_entry $map play "Attack" $curunit
-    adjust_menu_entry $map play "Overrun" $curunit
-    adjust_menu_entry $map play "Fire" $curunit
-    adjust_menu_entry $map play "Fire Into" $curunit
+    adjust_menu_entry $map play "Attack" [ expr $curunit && $can_attack ]
+    adjust_menu_entry $map play "Overrun" [ expr $curunit && $can_attack ]
+    adjust_menu_entry $map play "Fire" [ expr $curunit && $can_fire ]
+    adjust_menu_entry $map play "Fire Into" [ expr $curunit && $can_fire ]
     adjust_menu_entry $map play "Detonate" $curunit
 }
 
 proc adjust_more_menu { map } {
     global curunit
+    global can_act can_plan can_move can_return can_build can_attack can_fire
 
     adjust_menu_entry $map more "Give" $curunit
     adjust_menu_entry $map more "Take" $curunit
@@ -1285,8 +1371,10 @@ proc adjust_more_menu { map } {
     adjust_menu_entry $map more "Disembark" $curunit
     adjust_menu_entry $map more "Detach" $curunit
     adjust_menu_entry $map more "Disband" $curunit
-    adjust_menu_entry $map more "Plan Type" $curunit
-    adjust_menu_entry $map more "AI Control" $curunit
+    adjust_menu_entry $map more "Add Terrain" $curunit
+    adjust_menu_entry $map more "Remove Terrain" $curunit
+    adjust_menu_entry $map more "Plan Type" [ expr $curunit && $can_plan ]
+    adjust_menu_entry $map more "AI Control" [ expr $curunit && $can_plan ]
     adjust_menu_entry $map more "Rename..." $curunit
     adjust_menu_entry $map more "Give Unit" $curunit
 }
@@ -1318,6 +1406,8 @@ proc adjust_menu_entry { map menu entryname val } {
 }
 
 proc make_normal_bindings { map } {
+    global dside
+
     bind $map <Key> \
 	    { if { "%A" != "{}" } { handle_key_binding "%A" %W %X %Y } }
 
@@ -1326,10 +1416,10 @@ proc make_normal_bindings { map } {
     bind $map.leftside.mapf.map <Motion> { mouse_over_cmd %x %y }
     bind $map.leftside.mapf.map <Leave> { mouse_over_cmd -1 -1 }
 
-    set numu [ numutypes ]
+    set numu [ numutypes_available $dside ]
     set unitlist $map.rightside.listf.unitlist
-    for { set u 0 } { $u < $numu } { incr u } {
-	bind $unitlist.u$u <ButtonPress> [ list select_unit_type $u ]
+    for { set i 0 } { $i < $numu } { incr i } {
+	bind $unitlist.u$i <ButtonPress> [ list select_unit_type $i ]
     }
 
     bind $map.rightside.worldf.world <ButtonPress> \
@@ -1352,7 +1442,7 @@ proc handle_key_binding { str win x y } {
 }
 
 proc run_game_cmd {} {
-    set interval [ run_game 5 ]
+    set interval [ run_game 1 ]
     after $interval run_game_cmd
 }
 
@@ -1373,30 +1463,30 @@ proc update_game_state { str } {
 }
 
 proc update_game_side_info { s str everingame ingame status } {
-    set gamepane .m1.rightside.gamef.game
-    $gamepane itemconfigure s$s -text $str
+    set sidelist .m1.rightside.gamef.game
+    $sidelist itemconfigure s$s -text $str
     if { !$everingame } {
-	$gamepane itemconfigure s$s -fill gray
-	$gamepane itemconfigure frame$s -fill gray
+	$sidelist itemconfigure s$s -fill gray
+	$sidelist itemconfigure frame$s -fill gray
     }
     if { !$ingame } {
     }
     if { $status > 0 } {
 	# Find the the victory laurels and move into visibility.
-	set lis [ $gamepane coords won$s ]
+	set lis [ $sidelist coords won$s ]
 	set xval [ lindex $lis 0 ]
 	if { $xval > 4 } {
-	    $gamepane move won$s [expr 4 - $xval ] 0
-	    $gamepane raise won$s above all
+	    $sidelist move won$s [expr 4 - $xval ] 0
+	    $sidelist raise won$s above all
 	}
     }
     if { $status < 0 } {
 	# Find the the loss line and move into visibility.
-	set lis [ $gamepane coords lost$s ]
+	set lis [ $sidelist coords lost$s ]
 	set xval [ lindex $lis 0 ]
 	if { $xval > 0 } {
-	    $gamepane move lost$s [expr 0 - $xval ] 0
-	    $gamepane raise lost$s above all
+	    $sidelist move lost$s [expr 0 - $xval ] 0
+	    $sidelist raise lost$s above all
 	}
     }
 }
@@ -1408,10 +1498,9 @@ proc update_game_side_score { which str } {
 proc update_side_progress { s pri left resv } {
     global lineheight
 
-    set sy [ expr ($s - 1) * (2 * $lineheight + 20) ]
-    set tsy [ expr $sy + $lineheight ]
-    set rtop [ expr $sy + $lineheight + 4 + 1 ]
-    set rbot [ expr $sy + $lineheight + 12 ]
+    set old [ .m1.rightside.gamef.game coords left$s ]
+    set rtop [ lindex $old 1 ]
+    set rbot [ lindex $old 3 ]
     if { $pri == 0 } {
 	.m1.rightside.gamef.game itemconfig frame$s -outline gray
 	.m1.rightside.gamef.game coords left$s 24 $rtop 24 $rbot
@@ -1423,16 +1512,24 @@ proc update_side_progress { s pri left resv } {
     .m1.rightside.gamef.game coords resv$s 24 $rtop [ expr 24 + $resv ] $rbot
 }
 
-proc update_unitlist_name { u str } {
-    .m1.rightside.listf.unitlist itemconfigure u$u -text $str
+proc update_side_treasury { s j amt } {
+    .m1.rightside.gamef.game itemconfig m$s,$j -text $amt
 }
 
-proc update_unitlist_count { u str } {
-    .m1.rightside.listf.unitlist itemconfigure n$u -text $str
+proc update_unitlist_char { n str } {
+    .m1.rightside.listf.unitlist itemconfigure u$n -text $str
 }
 
-proc update_unitlist_incomplete { u str } {
-    .m1.rightside.listf.unitlist itemconfigure i$u -text $str
+proc update_unitlist_count { n str } {
+    .m1.rightside.listf.unitlist itemconfigure n$n -text $str
+}
+
+proc update_unitlist_incomplete { n str } {
+    .m1.rightside.listf.unitlist itemconfigure i$n -text $str
+}
+
+proc update_unitlist_name { n str } {
+    .m1.rightside.listf.unitlist itemconfigure name$n -text $str
 }
 
 proc update_unit_info { tag str } {
@@ -1454,14 +1551,14 @@ proc update_zoom { closer farther } {
 	set newstate disabled
     }
     .m1.menubar.view entryconfigure "Closer" -state $newstate
-    .m1.leftside.buttons.zoomin configure -state $newstate
+    .m1.leftside.buttons.zoomin config -state $newstate
     if { $farther != 0 } {
 	set newstate normal
     } else {
 	set newstate disabled
     }
     .m1.menubar.view entryconfigure "Farther" -state $newstate
-    .m1.leftside.buttons.zoomout configure -state $newstate
+    .m1.leftside.buttons.zoomout config -state $newstate
 }
 
 # Update the appearance of any mode controls/displays.
@@ -1499,6 +1596,30 @@ proc map_yscroll_set { first last } {
     .m1.leftside.mapf.yscroll set $first $last
 }
 
+proc update_action_controls_info { a1 a2 a3 a4 a5 a6 a7 } {
+    global can_act can_plan can_move can_return can_build can_attack can_fire
+
+    set can_act $a1
+    set can_plan $a2
+    set can_move $a3
+    set can_return $a4
+    set can_build $a5
+    set can_attack $a6
+    set can_fire $a7
+
+    if { "[ winfo exists .m1.leftside.buttons.build ]" } {
+	set state normal
+	if { !$can_build } {
+	    set state disabled
+	}
+	.m1.leftside.buttons.build config -state $state
+    }
+#    if { "[ winfo exists .m1.menubar.play ]" } {
+#	adjust_play_menu .m1
+#    }
+
+}
+
 proc update_view_controls_info { a1 a2 a3 a4 a5 a6 a7 a8 a9 } {
     global all_see_all
     global may_set_see_all
@@ -1594,20 +1715,20 @@ proc ask_unit_type_mode { str } {
     .m1.leftside.top.notices.t yview moveto 1.0
 }
 
-proc enable_unitlist { u flag } {
+proc enable_unitlist { n flag } {
     if { $flag == 1 } {
 	set color black
     } else {
 	set color [ .m1.rightside.gamef.game cget -background ]
     }
-    .m1.rightside.listf.unitlist itemconfigure rect$u -outline $color
+    .m1.rightside.listf.unitlist itemconfigure rect$n -outline $color
 }
 
-proc select_unit_type { u } {
-    set color [ .m1.rightside.listf.unitlist itemcget rect$u -outline ]
+proc select_unit_type { n } {
+    set color [ .m1.rightside.listf.unitlist itemcget rect$n -outline ]
     if { "$color" == "black" } {
-	set_unit_type $u
-	# Arguments are dummies, this is effecttively a pseudo-event
+	set_unit_type [ utype_actual $n ]
+	# Arguments are dummies, this is effectively a pseudo-event
 	# that gets the modalhandler function to run.
 	interp_key a 0 0
     }
@@ -1710,7 +1831,7 @@ proc create_agreements_window {} {
 # Create and popup the help window.
 
 proc create_help_window {} {
-    set textfont {-size 12}
+    global textfont
 
     toplevel .help
     wm title .help "Xconq Help"
