@@ -20,11 +20,16 @@ void abort ();
 #endif
 
 Pixmap tranpics[NUMPOWERS];
+Pixmap fuzzpics[NUMPOWERS];
 Pixmap bordpics[NUMPOWERS];
 Pixmap connpics[NUMPOWERS];
+Pixmap rrpics[NUMPOWERS];
 Pixmap windpics[5][NUMDIRS];
 Pixmap antpic;
 
+short utype_indexes[MAXUTYPES];
+short mtype_indexes[MAXMTYPES];
+
 static void set_optional_colors(void);
 static void init_cursors(void);
 static Tk_Cursor make_cursor(Tk_Window tkwin, char *cursbits, char *maskbits, int x, int y);
@@ -83,6 +88,9 @@ int min_h_for_unit_image = 16;
 
 #include "bitmaps/closer.b"
 #include "bitmaps/farther.b"
+#include "bitmaps/iso.b"
+#include "bitmaps/rotl.b"
+#include "bitmaps/rotr.b"
 
 #if 0
 #include "bitmaps/gray.b"
@@ -99,12 +107,22 @@ int min_h_for_unit_image = 16;
 #include "bitmaps/tran16.b"
 #include "bitmaps/tran32.b"
 
+#include "bitmaps/fuzz16.b"
+#include "bitmaps/fuzz32.b"
+#include "bitmaps/fuzz64.b"
+
 #include "bitmaps/bord16.b"
 #include "bitmaps/bord32.b"
 #include "bitmaps/bord64.b"
 
 #include "bitmaps/conn16.b"
 #include "bitmaps/conn32.b"
+#include "bitmaps/rr16.b"
+#include "bitmaps/rr32.b"
+
+#include "bitmaps/hex16iso.b"
+#include "bitmaps/hex32iso.b"
+#include "bitmaps/hex64iso.b"
 
 #include "bitmaps/wind0.b"
 #include "bitmaps/wind1ne.b"
@@ -225,7 +243,6 @@ init_display()
 	for_all_unit_types(u) {
 	    dside->ui->unitpics[p][u] = None;
 	    dside->ui->unitmasks[p][u] = None;
-	    dside->ui->unitchars[p][u] = '\0';
 #if 0
 	    dside->ui->ulegendfonts[p][u] = NULL;
 #endif
@@ -250,6 +267,27 @@ init_display()
     init_cursors();
     update_view_controls_info();
 
+    /* Compute the position of each unit type in the list of unit types. */
+    {
+	int i = 0;
+
+	for_all_unit_types(u) {
+	    if (type_ever_available(u, dside))
+	      utype_indexes[u] = i++;
+	    else
+	      utype_indexes[u] = -1;
+	}
+    }
+    {
+	int m, i = 0;
+
+	for_all_material_types(m) {
+	    if (m_treasury(m))
+	      mtype_indexes[m] = i++;
+	    else
+	      mtype_indexes[m] = -1;
+	}
+    }
     /* Create the generic windows. */
     dside->ui->maps = NULL;
 
@@ -272,7 +310,7 @@ init_display()
 void
 set_colors()
 {
-    int	t;
+    int	t, i;
 
     dside->ui->fgcolor = request_color("black");
     dside->ui->bgcolor = request_color("white");
@@ -294,6 +332,8 @@ set_colors()
     /* Set colors to distinctive default; will set usefully later. */
     for_all_terrain_types(t) {
 	dside->ui->cellcolor[t] = NULL;
+	for (i = 0; i < 5; ++i)
+	  dside->ui->cellshades[t][i] = dside->ui->graycolor;
     }
 }
 
@@ -556,6 +596,19 @@ init_bitmaps()
     tranpics[5] =
       Tk_GetBitmap(interp, rootwin, Tk_GetUid("tran32"));
 
+    Tk_DefineBitmap(interp, Tk_GetUid("fuzz16"),
+		    fuzz16_bits, fuzz16_width, fuzz16_height);
+    fuzzpics[4] =
+      Tk_GetBitmap(interp, rootwin, Tk_GetUid("fuzz16"));
+    Tk_DefineBitmap(interp, Tk_GetUid("fuzz32"),
+		    fuzz32_bits, fuzz32_width, fuzz32_height);
+    fuzzpics[5] =
+      Tk_GetBitmap(interp, rootwin, Tk_GetUid("fuzz32"));
+    Tk_DefineBitmap(interp, Tk_GetUid("fuzz64"),
+		    fuzz64_bits, fuzz64_width, fuzz64_height);
+    fuzzpics[6] =
+      Tk_GetBitmap(interp, rootwin, Tk_GetUid("fuzz64"));
+
 
     Tk_DefineBitmap(interp, Tk_GetUid("bord16"),
 		    bord16_bits, bord16_width, bord16_height);
@@ -579,6 +632,28 @@ init_bitmaps()
     connpics[5] =
       Tk_GetBitmap(interp, rootwin, Tk_GetUid("conn32"));
 
+    Tk_DefineBitmap(interp, Tk_GetUid("rr16"),
+		    rr16_bits, rr16_width, rr16_height);
+    rrpics[4] =
+      Tk_GetBitmap(interp, rootwin, Tk_GetUid("rr16"));
+    Tk_DefineBitmap(interp, Tk_GetUid("rr32"),
+		    rr32_bits, rr32_width, rr32_height);
+    rrpics[5] =
+      Tk_GetBitmap(interp, rootwin, Tk_GetUid("rr32"));
+
+    Tk_DefineBitmap(interp, Tk_GetUid("hex16iso"),
+		    hex16iso_bits, hex16iso_width, hex16iso_height);
+    dside->ui->hexisopics[4] =
+      Tk_GetBitmap(interp, rootwin, Tk_GetUid("hex16iso"));
+    Tk_DefineBitmap(interp, Tk_GetUid("hex32iso"),
+		    hex32iso_bits, hex32iso_width, hex32iso_height);
+    dside->ui->hexisopics[5] =
+      Tk_GetBitmap(interp, rootwin, Tk_GetUid("hex32iso"));
+    Tk_DefineBitmap(interp, Tk_GetUid("hex64iso"),
+		    hex64iso_bits, hex64iso_width, hex64iso_height);
+    dside->ui->hexisopics[6] =
+      Tk_GetBitmap(interp, rootwin, Tk_GetUid("hex64iso"));
+
     Tk_DefineBitmap(interp, Tk_GetUid("wind0"),
 		    wind0_bits, wind0_width, wind0_height);
     for_all_directions(dir) {
@@ -693,6 +768,12 @@ init_bitmaps()
 		    closer_bits, closer_width, closer_height);
     Tk_DefineBitmap(interp, Tk_GetUid("farther"),
 		    farther_bits, farther_width, farther_height);
+    Tk_DefineBitmap(interp, Tk_GetUid("iso"),
+		    iso_bits, iso_width, iso_height);
+    Tk_DefineBitmap(interp, Tk_GetUid("rotl"),
+		    rotl_bits, rotl_width, rotl_height);
+    Tk_DefineBitmap(interp, Tk_GetUid("rotr"),
+		    rotr_bits, rotr_width, rotr_height);
     Tk_DefineBitmap(interp, Tk_GetUid("laurels"),
 		    laurels_bits, laurels_width, laurels_height);
 }
@@ -769,13 +850,45 @@ init_terrain_images()
 	    if (timg->hook) {
 		tkimg = (TkImage *) timg->hook;
 		/* Save the color if found. */
-		if (tkimg->solid) 
+		if (tkimg->solid)
 		  dside->ui->cellcolor[t] = tkimg->solid;
 	    }
 	}
+	if (elevations_defined())
+	  init_shades(t);
     }
 }
 
+init_shades(int t)
+{
+    XColor col, *newcolor, *basecolor = dside->ui->cellcolor[t];
+    Tk_Window rootwin = Tk_MainWindow(interp);
+
+    if (basecolor == NULL)
+      basecolor = dside->ui->graycolor;
+    col.red   = min((12 * basecolor->red) / 10, 65535);
+    col.green = min((12 * basecolor->green) / 10, 65535);
+    col.blue  = min((12 * basecolor->blue) / 10, 65535);
+    if ((newcolor = Tk_GetColorByValue(rootwin, &col)) != NULL)
+      dside->ui->cellshades[t][0] = newcolor;
+    col.red   = min((11 * basecolor->red) / 10, 65535);
+    col.green = min((11 * basecolor->green) / 10, 65535);
+    col.blue  = min((11 * basecolor->blue) / 10, 65535);
+    if ((newcolor = Tk_GetColorByValue(rootwin, &col)) != NULL)
+      dside->ui->cellshades[t][1] = newcolor;
+    dside->ui->cellshades[t][2] = basecolor;
+    col.red   = (9 * basecolor->red) / 10;
+    col.green = (9 * basecolor->green) / 10;
+    col.blue  = (9 * basecolor->blue) / 10;
+    if ((newcolor = Tk_GetColorByValue(rootwin, &col)) != NULL)
+      dside->ui->cellshades[t][3] = newcolor;
+    col.red   = (8 * basecolor->red) / 10;
+    col.green = (8 * basecolor->green) / 10;
+    col.blue  = (8 * basecolor->blue) / 10;
+    if ((newcolor = Tk_GetColorByValue(rootwin, &col)) != NULL)
+      dside->ui->cellshades[t][4] = newcolor;
+}
+
 /* Set up a side's view of everybody else's colors and emblems. */
 
 static void
@@ -892,3 +1005,44 @@ update_view_controls_info()
 		 winds_defined(),
 		 clouds_defined());
 }
+
+
+update_action_controls_info(Map *map)
+{
+    int canact = FALSE, canplan = FALSE, canmove = FALSE, canreturn = FALSE;
+    int canbuild = FALSE, canrepair = FALSE;
+    int canattack = FALSE, canfire = FALSE;
+    Unit *unit;
+
+    unit = map->curunit;
+    if (in_play(unit)) {
+	if (u_acp(unit->type) > 0)
+	  canact = TRUE;
+	if (unit->plan)
+	  canplan = TRUE;
+	if (can_move_at_all(unit)) {
+	    canmove = TRUE;
+	    if (nummtypes > 0)
+	      canreturn = TRUE;
+	}
+	if ((can_create(unit) || can_complete(unit))
+	    && (unit->transport == NULL
+		|| !completed(unit->transport)
+		|| uu_occ_can_build(unit->transport->type, unit->type))
+	    || can_develop(unit)) {
+	    canbuild = TRUE;
+	}
+	if (can_repair(unit)) {
+	    canrepair = TRUE;
+	}
+	if (can_attack(unit)) {
+	    canattack = TRUE;
+	}
+	if (can_fire(unit)) {
+	    canfire = TRUE;
+	}
+    }
+    eval_tcl_cmd("update_action_controls_info %d %d %d %d %d %d %d",
+		 canact, canplan, canmove, canreturn,
+		 canbuild, canattack, canfire);
+}
