@@ -11,6 +11,8 @@ extern void notify_instructions(void);
 extern int sendnow;
 #include "kpublic.h"
 #include "tkconq.h"
+extern short utype_indexes[];
+extern short mtype_indexes[];
 
 void place_legends(Side *side);
 
@@ -52,6 +54,8 @@ int tk_numutypes(ClientData cldata, Tcl_Interp *interp,
 int tk_numsides(ClientData cldata, Tcl_Interp *interp, int argc, char *argv[]);
 int tk_maxsides(ClientData cldata, Tcl_Interp *interp, int argc, char *argv[]);
 int tk_dside(ClientData cldata, Tcl_Interp *interp, int argc, char *argv[]);
+int tk_numtreasury(ClientData cldata, Tcl_Interp *interp,
+		   int argc, char *argv[]);
 int tk_numfeatures(ClientData cldata, Tcl_Interp *interp,
 		   int argc, char *argv[]);
 int tk_game_info(ClientData cldata, Tcl_Interp *interp,
@@ -64,6 +68,8 @@ int tk_utype_name(ClientData cldata, Tcl_Interp *interp,
 		  int argc, char *argv[]);
 int tk_u_image_name(ClientData cldata, Tcl_Interp *interp,
 		    int argc, char *argv[]);
+int tk_mtype_name(ClientData cldata, Tcl_Interp *interp,
+		  int argc, char *argv[]);
 int tk_side_name(ClientData cldata, Tcl_Interp *interp,
 		 int argc, char *argv[]);
 int tk_side_adjective(ClientData cldata, Tcl_Interp *interp,
@@ -132,6 +138,13 @@ int mapw_cmd(ClientData cldata, Tcl_Interp *interp, int argc, char *argv[]);
 int imfsample_cmd(ClientData cldata, Tcl_Interp *interp,
 		  int argc, char *argv[]);
 
+int tk_numutypes_available(ClientData cldata, Tcl_Interp *interp,
+			   int argc, char *argv[]);
+int tk_utype_actual(ClientData cldata, Tcl_Interp *interp,
+			   int argc, char *argv[]);
+int tk_mtype_actual(ClientData cldata, Tcl_Interp *interp,
+			   int argc, char *argv[]);
+
 void update_mouseover(Map *map, int rawx, int rawy);
 
 /* Create the one global interpreter, add Xconq-specific commands to it. */
@@ -185,6 +198,8 @@ initial_ui_init()
 		      (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
     Tcl_CreateCommand(interp, "maxsides", tk_maxsides,
 		      (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+    Tcl_CreateCommand(interp, "numtreasury", tk_numtreasury,
+		      (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
     Tcl_CreateCommand(interp, "numfeatures", tk_numfeatures,
 		      (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
 
@@ -198,6 +213,8 @@ initial_ui_init()
 		      (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
     Tcl_CreateCommand(interp, "u_image_name", tk_u_image_name,
 		      (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+    Tcl_CreateCommand(interp, "mtype_name", tk_mtype_name,
+		      (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
     Tcl_CreateCommand(interp, "side_name", tk_side_name,
 		      (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
     Tcl_CreateCommand(interp, "side_adjective", tk_side_adjective,
@@ -273,6 +290,13 @@ initial_ui_init()
     Tcl_CreateCommand(interp, "designer_save", tk_designer_save,
 		      (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
 
+    Tcl_CreateCommand(interp, "numutypes_available", tk_numutypes_available,
+		      (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+    Tcl_CreateCommand(interp, "utype_actual", tk_utype_actual,
+		      (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+    Tcl_CreateCommand(interp, "mtype_actual", tk_mtype_actual,
+		      (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
+
     tkwin = Tk_MainWindow(interp);
 
     Tcl_CreateCommand(interp, "map", mapw_cmd,
@@ -408,6 +432,19 @@ tk_numfeatures(ClientData cldata, Tcl_Interp *interp, int argc, char *argv[])
     return TCL_OK;
 }
 
+int
+tk_numtreasury(ClientData cldata, Tcl_Interp *interp, int argc, char *argv[])
+{
+    int m, rslt = 0;
+
+    for_all_material_types(m) {
+	if (m_treasury(m))
+	  ++rslt;
+    }
+    sprintf(interp->result, "%d", rslt);
+    return TCL_OK;
+}
+
 int
 tk_ttype_name(ClientData cldata, Tcl_Interp *interp, int argc, char *argv[])
 {
@@ -468,6 +505,19 @@ tk_u_image_name(ClientData cldata, Tcl_Interp *interp, int argc, char *argv[])
     return TCL_OK;
 }
 
+int
+tk_mtype_name(ClientData cldata, Tcl_Interp *interp, int argc, char *argv[])
+{
+    int m;
+
+    m = strtol(argv[1], NULL, 10);
+    if (is_material_type(m))
+      sprintf(interp->result, "%s", m_type_name(m));
+    else
+      sprintf(interp->result, "?m?");
+    return TCL_OK;
+}
+
 int
 tk_side_name(ClientData cldata, Tcl_Interp *interp, int argc, char *argv[])
 {
@@ -809,6 +859,11 @@ tk_run_game(ClientData cldata, Tcl_Interp *interp, int argc, char *argv[])
     int maxactions, rslt = 0;
     unsigned long interval;
 
+    if (argc != 2) {
+	run_warning("wrong # args (%d) to run_game", argc - 1);
+	return TCL_OK;
+    }
+
     maxactions = strtol(argv[1], NULL, 10);
 
     /* Check for any input from remotes. */
@@ -964,7 +1019,7 @@ update_mouseover(Map *map, int rawx, int rawy)
 {
     DGprintf("over %d %d\n", rawx, rawy);
     tmpbuf[0] = '\0';
-    if (rawx >= 0 || rawy >= 0)
+    if ((rawx >= 0 || rawy >= 0) && map->widget != NULL)
       oneliner(dside, widget_vp(map), rawx, rawy);
     eval_tcl_cmd("update_mouseover \"%s\"", tmpbuf);
 }
@@ -1015,6 +1070,7 @@ set_current_unit(Map *map, Unit *unit)
     if (map->curunit)
       update_at_unit(map, map->curunit);
     draw_unit_info(map);
+    update_action_controls_info(map);
 }
 
 draw_unit_info(Map *map)
@@ -1075,7 +1131,7 @@ draw_unit_info(Map *map)
     eval_tcl_cmd("update_unit_info plan \"%s\"", infobuf);
     task = (unit->plan ? unit->plan->tasks : NULL);
     for (i = 0; i < 2; ++i) {
-	task_desc(infobuf, unit->side, task);
+	task_desc(infobuf, unit->side, unit, task);
 	eval_tcl_cmd("update_unit_info t%d \"%s\"", i, infobuf);
 	if (task)
 	  task = task->next;
@@ -1105,12 +1161,30 @@ update_area_display(Side *side)
 void
 update_cell_display(Side *side, int x, int y, int flags)
 {
+    int dir, x1, y1;
     Map *map;
+    VP *vp;
 
     if (!active_display(side))
       return;
-    for_all_maps(map)
-      update_at_cell(map, x, y, flags);
+    for_all_maps(map) {
+	update_at_cell(map, x, y, flags);
+	if (flags & UPDATE_ADJ) {
+	    if (side->terrview != NULL) {
+		vp = widget_vp(map);
+		if (vp->hw <= 10)
+		  continue;
+		for_all_directions(dir) {
+		    if (point_in_dir(x, y, dir, &x1, &y1)
+			/* A totally unseen adjacent cell will have
+			   nothing worth redrawing. */
+			&& terrain_view(dside, x1, y1) != UNSEEN) {
+			update_at_cell(map, x1, y1, flags);
+		    }
+		}
+	    }
+	}
+    }
 }
 
 /* The kernel calls this to update info about the given side. */
@@ -1118,7 +1192,7 @@ update_cell_display(Side *side, int x, int y, int flags)
 void
 update_side_display(Side *side, Side *side2, int rightnow)
 {
-    int sx, sy, totacp, percentleft, percentresv, activ;
+    int sx, sy, totacp, percentleft, percentresv, activ, m;
     char sidebuf[BUFSIZE];
     extern int curpriority;
 
@@ -1171,6 +1245,13 @@ update_side_display(Side *side, Side *side2, int rightnow)
     activ = (!g_use_side_priority() || curpriority == side2->priority);
     eval_tcl_cmd("update_side_progress %d %d %d %d",
 		 side_number(side2), activ, percentleft, percentresv);
+    for_all_material_types(m) {
+	if (m_treasury(m)) {
+	    eval_tcl_cmd("update_side_treasury %d %d %d",
+			 side_number(side2), mtype_indexes[m],
+			 side2->treasury[m]);
+	}
+    }
 }
 
 /* The kernel calls this to update info about the given unit. */
@@ -1185,10 +1266,12 @@ update_unit_display(Side *side, Unit *unit, int rightnow)
     if (unit == NULL)
       return;
     if (inside_area(unit->x, unit->y)) {
-	update_cell_display(side, unit->x, unit->y, rightnow);
+	update_cell_display(side, unit->x, unit->y, UPDATE_ALWAYS);
+    }
+    for_all_maps(map) {
+	draw_unit_info(map);
+	update_action_controls_info(map);
     }
-    for_all_maps(map)
-      draw_unit_info(map);
     update_side_display(side, unit->side, rightnow);
     update_unit_type_list(side, unit->type);
 }
@@ -1250,6 +1333,14 @@ update_turn_display(Side *side, int rightnow)
 void
 update_action_display(Side *side, int rightnow)
 {
+    Map *map;
+
+    if (!active_display(side))
+      return;
+
+    for_all_maps(map) {
+	update_action_controls_info(map);
+    }
 }
 
 void
@@ -1489,25 +1580,27 @@ update_unit_type_list(Side *side, int u)
 
     if (!between(0, u, numutypes))
       return;
+    i = utype_indexes[u];
     /* Only show a char for the unit type if all have single chars,
        otherwise blank all. */
-    eval_tcl_cmd("update_unitlist_name %d \"%c\"",
-		 u, (longest_shortest == 1 ? *(shortest_unique_name(u)) : ' '));
+    eval_tcl_cmd("update_unitlist_char %d \"%c\"",
+		 i, (longest_shortest == 1 ? *(shortest_unique_name(u)) : ' '));
     /* Our unit total (right-justified) */
     spbuf[0] = '\0';
     num = num_units_in_play(side, u);
     if (num > 0) {
 	sprintf(spbuf, "%d", num);
     }
-    eval_tcl_cmd("update_unitlist_count %d \"%s\"", u, spbuf);
+    eval_tcl_cmd("update_unitlist_count %d \"%s\"", i, spbuf);
     /* Our units under construction (left-justified) */
     num = num_units_incomplete(side, u);
     spbuf[0] = '\0';
     if (num > 0) {
 	sprintf(spbuf, "(%d)", num);
     }
-    eval_tcl_cmd("update_unitlist_incomplete %d \"%s\"", u, spbuf);
+    eval_tcl_cmd("update_unitlist_incomplete %d \"%s\"", i, spbuf);
     /* (should do other columns too) */
+    eval_tcl_cmd("update_unitlist_name %d \"%s\"", i, u_type_name(u));
 }
 
 popup_help(Side *side, HelpNode *node)
@@ -1635,7 +1728,7 @@ tk_interp_key(ClientData cldata, Tcl_Interp *interp, int argc, char *argv[])
     tkwin = Tk_CoordsToWindow(rawx, rawy, Tk_MainWindow(interp));
     map->inpsx = map->inpsy = -1;
     map->inpx = map->inpy = -1;
-    if (tkwin) {
+    if (tkwin && map->widget != NULL) {
 	Tk_GetRootCoords(tkwin, &winx, &winy);
 	map->inpsx = rawx - winx;  map->inpsy = rawy - winy;
 	if (nearest_cell(widget_vp(map), map->inpsx, map->inpsy,
@@ -1706,8 +1799,20 @@ tk_set_design_tool(ClientData cldata, Tcl_Interp *interp,
     if (strcmp(arg, "normal") == 0) {
 	dside->ui->curdesigntool = survey_mode;
     } else if (strcmp(arg, "terrain") == 0) {
-	/* (should choose subtype) */
-	dside->ui->curdesigntool = cell_paint_mode;
+	switch (t_subtype(dside->ui->curttype)) {
+	  case bordersubtype:
+	    dside->ui->curdesigntool = bord_paint_mode;
+	    break;
+	  case connectionsubtype:
+	    dside->ui->curdesigntool = conn_paint_mode;
+	    break;
+	  case coatingsubtype:
+	    dside->ui->curdesigntool = coat_paint_mode;
+	    break;
+	  default:
+	    dside->ui->curdesigntool = cell_paint_mode;
+	    break;
+	}
     } else if (strcmp(arg, "unit") == 0) {
 	dside->ui->curdesigntool = unit_paint_mode;
     } else if (strcmp(arg, "people") == 0) {
@@ -1774,6 +1879,20 @@ tk_set_design_data(ClientData cldata, Tcl_Interp *interp,
 	    val = dside->ui->curttype;
 	}
 	dside->ui->curttype = val;
+	switch (t_subtype(dside->ui->curttype)) {
+	  case bordersubtype:
+	    dside->ui->curdesigntool = bord_paint_mode;
+	    break;
+	  case connectionsubtype:
+	    dside->ui->curdesigntool = conn_paint_mode;
+	    break;
+	  case coatingsubtype:
+	    dside->ui->curdesigntool = coat_paint_mode;
+	    break;
+	  default:
+	    dside->ui->curdesigntool = cell_paint_mode;
+	    break;
+	}
     } else if (strcmp(type, "curbgttype") == 0) {
 	dside->ui->curbgttype = val;
     } else if (strcmp(type, "curutype") == 0) {
@@ -1876,6 +1995,55 @@ tk_designer_save(ClientData cldata, Tcl_Interp *interp,
     return TCL_OK;
 }
 
+int
+tk_numutypes_available(ClientData cldata, Tcl_Interp *interp,
+		       int argc, char *argv[])
+{
+    int num, u;
+
+    num = 0;
+    for_all_unit_types(u) {
+	if (utype_indexes[u] >=0)
+	  ++num;
+    }
+    sprintf(interp->result, "%d", num);
+    return TCL_OK;
+}
+
+int
+tk_utype_actual(ClientData cldata, Tcl_Interp *interp, int argc, char *argv[])
+{
+    int n, u;
+
+    n = strtol(argv[1], NULL, 10);
+    for_all_unit_types(u) {
+	if (utype_indexes[u] == n) {
+	    sprintf(interp->result, "%d", u);
+	    return TCL_OK;
+	}
+    }
+    /* (should make error) */
+    sprintf(interp->result, "%d", -1);
+    return TCL_OK;
+}
+
+int
+tk_mtype_actual(ClientData cldata, Tcl_Interp *interp, int argc, char *argv[])
+{
+    int n, m;
+
+    n = strtol(argv[1], NULL, 10);
+    for_all_material_types(m) {
+	if (mtype_indexes[m] == n) {
+	    sprintf(interp->result, "%d", m);
+	    return TCL_OK;
+	}
+    }
+    /* (should make error) */
+    sprintf(interp->result, "%d", -1);
+    return TCL_OK;
+}
+
 void
 place_legends(side)
 Side *side;
@@ -2354,7 +2522,7 @@ Side *side;
 Map *map;
 int u, flag;
 {
-    eval_tcl_cmd("enable_unitlist %d %d", u, flag);
+    eval_tcl_cmd("enable_unitlist %d %d", utype_indexes[u], flag);
 }
 
 int
