@@ -11,8 +11,10 @@ any later version.  See the file COPYING.  */
 #include "tkconq.h"
 
 extern Pixmap tranpics[NUMPOWERS];
+extern Pixmap fuzzpics[NUMPOWERS];
 extern Pixmap bordpics[NUMPOWERS];
 extern Pixmap connpics[NUMPOWERS];
+extern Pixmap rrpics[NUMPOWERS];
 extern Pixmap windpics[5][NUMDIRS];
 extern Pixmap antpic;
 
@@ -91,7 +93,11 @@ static int mapw_widget_cmd(ClientData cldata, Tcl_Interp *interp,
 			   int argc, char **argv);
 static void mapw_display(ClientData cldata);
 static void draw_map_widget(MapW *mapw);
+static void draw_map_overhead(MapW *mapw);
+static void draw_map_isometric(MapW *mapw);
 
+static void draw_unseen_fuzz(MapW *mapw, int vx, int vy, int vw, int y1,
+			     int y2);
 static void draw_row(MapW *mapw, int x0, int y0, int len);
 static void draw_current(MapW *mapw);
 
@@ -413,6 +419,7 @@ int flags;
 	mapw->gc = XCreateGC(mapw->display, DefaultRootWindow(mapw->display),
 			     None, NULL);
 	XSetTSOrigin(mapw->display, mapw->gc, 0, 0);
+	XSetLineAttributes(mapw->display, mapw->gc, 0, LineSolid, CapButt, JoinMiter);
     }
     if (mapw->main_font == NULL)
       mapw->main_font = Tk_GetFont(interp, mapw->tkwin, "fixed");
@@ -721,7 +728,6 @@ draw_area_background(MapW *mapw)
     color = (1 /* grid color matches unseen color */ ? dside->ui->grid_color : dside->ui->unseen_color);
     XSetClipMask(dpy, gc, None);
     XSetForeground(dpy, gc, color->pixel);
-    XSetLineAttributes(dpy, gc, 1, LineSolid, CapButt, JoinMiter);
 
     if (area.xwrap) {
 	/* Area is cylinder; draw a rectangle. */
@@ -743,7 +749,7 @@ draw_area_background(MapW *mapw)
 	points[1].x = lrx;  points[1].y = lry;
 	xform(mapw, aw - 1, ah / 2, &rx, &ry);
 	points[2].x = rx;   points[2].y = ry;
-	xform(mapw, aw - 1 - ah / 2, ah - 1, &urx, &ury);
+	xform(mapw, aw - 1 - (ah - 1) / 2, ah - 1, &urx, &ury);
 	points[3].x = urx;  points[3].y = ury;
 	xform(mapw, 0, ah - 1, &ulx, &uly);
 	points[4].x = ulx;  points[4].y = uly;
@@ -763,13 +769,28 @@ draw_area_background(MapW *mapw)
 
 /* Draw the view proper. */
 
-void
+static void
 draw_map_widget(MapW *mapw)
 {
-    int y1, y2, y, x1, vx, vy, vw, vh, len, t;
-
     draw_area_background(mapw);
 
+    if (mapw->vp->isometric)
+      draw_map_isometric(mapw);
+    else
+      draw_map_overhead(mapw);
+
+    if (mapw->map->curunit)
+      draw_current(mapw);
+    if (mapw->blasttype >= 0)
+      draw_blast_image(mapw, mapw->blastsx - mapw->rsx, mapw->blastsy - mapw->rsy,
+		       mapw->blastsw, mapw->blastsh, mapw->blasttype);
+}
+
+static void
+draw_map_overhead(MapW *mapw)
+{
+    int y1, y2, y, x1, vx, vy, vw, vh, len, t;
+
     if (mapw->vp->vcx < 0 || mapw->vp->vcy < 0) {
     	run_warning("doing a nasty hack");
 	mapw->vp->vcx = mapw->vp->vcy = 2;
@@ -828,6 +849,9 @@ draw_map_widget(MapW *mapw)
 		draw_contours(mapw, x1, y, len);
 	    }
 	}
+	/* Fuzz out the edges of the known area. */
+	if (!mapw->map->see_all && between(4, mapw->vp->power, 6))
+	  draw_unseen_fuzz(mapw, vx, vy, vw, y1, y2);
     }
     /* Now draw the lat-long grid if asked to do so. */
     if (mapw->vp->draw_latlong && mapw->vp->latlong_interval > 0)
@@ -837,11 +861,6 @@ draw_map_widget(MapW *mapw)
 	  continue;
 	draw_row(mapw, x1, y, len);
     }
-    if (mapw->map->curunit)
-      draw_current(mapw);
-    if (mapw->blasttype >= 0)
-      draw_blast_image(mapw, mapw->blastsx - mapw->rsx, mapw->blastsy - mapw->rsy,
-		       mapw->blastsw, mapw->blastsh, mapw->blasttype);
 }
 
 /* Given a row in the viewport, compute the starting cell and length
@@ -888,6 +907,226 @@ compute_x1_len(MapW *mapw, int vx, int vy, int vw, int y, int *x1p, int *lenp)
     return (*lenp > 0);
 }
 
+static void
+draw_map_isometric(MapW *mapw)
+{
+    int x0, y0, x, y, xw, sx, sy, t;
+    int i, j, dx1a1, dy1a1, dx1a2, dy1a2, dx1b1, dy1b1, dx1b2, dy1b2, firstpart, dx2, dy2;
+    int xfirstend, yfirstend, zigzaga, zigzagb, alternate;
+    int over;
+    XColor *color;
+    enum whattouse drawmeth;
+    Pixmap pat;
+    Image *timg;
+    GC gc = mapw->gc;
+    Display *dpy = mapw->display;
+    enum grayshade shade;
+
+    if (mapw->vp->power >= 3) {
+	switch (mapw->vp->isodir) {
+	  case NORTHEAST:
+	    x0 = area.width - 1;  y0 = area.height - 1;
+	    zigzaga = FALSE;
+	    dx1a1 = -1;  dy1a1 =  0;
+	    xfirstend = 0;  yfirstend = area.height - 1;
+	    zigzagb = TRUE;
+	    dx1b1 =  1;  dy1b1 = -1;
+	    dx1b2 = -1;  dy1b2 = 0;
+	    dx2 = 2;  dy2 = -1;
+	    break;
+	  case EAST:
+	    x0 = area.width - 1;  y0 = area.height - 1;
+	    zigzaga = TRUE;
+	    dx1a1 =  0;  dy1a1 = -1;
+	    dx1a2 = -1;  dy1a2 =  1;
+	    xfirstend = 0;  yfirstend = area.height - 1;
+	    zigzagb = FALSE;
+	    dx1b1 =  0;  dy1b1 = -1;
+	    dx2 = 1;  dy2 = -2;
+	    break;
+	  case SOUTHEAST:
+	    x0 = area.width - 1;  y0 = 0;
+	    zigzaga = FALSE;
+	    dx1a1 = 0;  dy1a1 = 1;
+	    xfirstend = 0;  yfirstend = area.height - 1;
+	    zigzagb = FALSE;
+	    dx1b1 = -1;  dy1b1 = 0;
+	    dx2 = -1;  dy2 = -1;
+	    break;
+	  case SOUTHWEST:
+	    x0 = 0;  y0 = 0;
+	    zigzaga = FALSE;
+	    dx1a1 = 1;  dy1a1 = 0;
+	    xfirstend = area.width - 1;  yfirstend = 0;
+	    zigzagb = TRUE;
+	    dx1b1 = -1;  dy1b1 = 1;
+	    dx1b2 =  1;  dy1b2 = 0;
+	    dx2 = -2;  dy2 = 1;
+	    break;
+	  case WEST:
+	    x0 = 0;  y0 = 0;
+	    zigzaga = TRUE;
+	    dx1a1 = 0;  dy1a1 =  1;
+	    dx1a2 = 1;  dy1a2 = -1;
+	    xfirstend = area.width - 1;  yfirstend = 0;
+	    zigzagb = FALSE;
+	    dx1b1 = 0;  dy1b1 = 1;
+	    dx2 = -1;  dy2 = 2;
+	    break;
+	  case NORTHWEST:
+	    x0 = 0;  y0 = area.height - 1;
+	    zigzaga = FALSE;
+	    dx1a1 = 0;  dy1a1 = -1;
+	    xfirstend = 0;  yfirstend = 0;
+	    zigzagb = FALSE;
+	    dx1b1 = 1;  dy1b1 = 0;
+	    dx2 = 1;  dy2 = 1;
+	    break;
+	  default:
+	    break;
+	}
+	firstpart = TRUE;
+	alternate = FALSE;
+	for (i = 0; i < 2 * (area.width + area.height); ++i) {
+	    x = x0;  y = y0;
+	    for (j = 0; j < (area.width + area.height) / 2; ++j) {
+		if (in_area(x, y)) {
+		    xw = wrapx(x);
+		    drawmeth = cell_drawing_info(mapw, xw, y, &pat, &color, &over);
+		    if (drawmeth != dontdraw) {
+			XSetForeground(dpy, gc, color->pixel);
+			draw_terrain_iso(mapw, x, y);
+			draw_row(mapw, x, y, 1);
+		    }
+		}
+		x += dx2;  y += dy2;
+	    }
+	    if (x == xfirstend && y == yfirstend)
+	      firstpart = FALSE;
+	    if (firstpart) {
+		if (zigzaga) {
+		    if (!alternate) {
+			x0 += dx1a1;  y0 += dy1a1;
+		    } else {
+			x0 += dx1a2;  y0 += dy1a2;
+		    }
+		    alternate = !alternate;
+		} else {
+		    x0 += dx1a1;  y0 += dy1a1;
+		}
+	    } else {
+		if (zigzagb) {
+		    if (!alternate) {
+			x0 += dx1b1;  y0 += dy1b1;
+		    } else {
+			x0 += dx1b2;  y0 += dy1b2;
+		    }
+		    alternate = !alternate;
+		} else {
+		    x0 += dx1b1;  y0 += dy1b1;
+		}
+	    }
+	}
+    } else {
+	/* (draw columns) */
+    }
+    XSetFillStyle(dpy, gc, FillSolid);
+}
+
+draw_terrain_iso(MapW *mapw, int x, int y)
+{
+    int xw, t, sx, sy, elev, elev1, drop, drawcliffs, x1, x1w, y1, sx1, sy1, t1, dir;
+    int viewdir = mapw->vp->isodir, over;
+    int hw = mapw->vp->hw, hh = mapw->vp->hh;
+    Image *timg;
+    GC gc = mapw->gc;
+    Display *dpy = mapw->display;
+    extern int any_thickness;
+    XPoint points[4];
+
+    xw = wrapx(x);
+    t = terrain_at(xw, y);
+    timg = best_image(dside->ui->timages[t], hw, hh);
+    XSetClipMask(dpy, gc, dside->ui->hexisopics[mapw->vp->power]);
+    set_terrain_gc_for_image(mapw, gc, timg);
+    xform(mapw, x, y, &sx, &sy);
+    XSetClipOrigin(dpy, gc, sx, sy);
+    XFillRectangle(dpy, mapw->d, gc, sx, sy, hh, hw / 2);
+    if (elevations_defined() || any_thickness) {
+	elev = (elevations_defined() ? elev_at(xw, y) : 0) + t_thickness(t);
+	XSetClipMask(dpy, gc, None);
+	XSetFillStyle(dpy, gc, FillSolid);
+	for_all_directions(dir) {
+	    drawcliffs = FALSE;
+	    XSetForeground(dpy, gc,
+			   dside->ui->cellshades[t][shading(dir)]->pixel);
+	    if (point_in_dir(xw, y, dir, &x1, &y1)) {
+		t1 = terrain_at(x1, y1);
+		x1w = wrapx(x1);
+		elev1 = ((elevations_defined() ? elev_at(x1w, y1) : 0)
+			 + t_thickness(t1));
+		drop = elev - elev1;
+		if (drop > 0 /* (should be "> one pixel") */) {
+		    xform(mapw, x1, y1, &sx1, &sy1);
+		    if (dir == opposite_dir(viewdir)) {
+			points[0].x = sx + hw / 4;  points[0].y = sy + hw / 2;
+			points[1].x = sx + hh - hw / 4;  points[1].y = sy + hw / 2;
+			points[2].x = sx + hh - hw / 4;  points[2].y = sy1;
+			points[3].x = sx + hw / 4;  points[3].y = sy1;
+			XFillPolygon(dpy, mapw->d, gc, points, 4,
+				     Convex, CoordModeOrigin);
+		    } else if (dir == left_dir(opposite_dir(viewdir))) {
+			points[0].x = sx + hh - hw / 4;  points[0].y = sy + hw / 2;
+			points[1].x = sx + hh;  points[1].y = sy + hw / 4;
+			points[2].x = sx + hh;  points[2].y = sy1;
+			points[3].x = sx + hh - hw / 4;  points[3].y = sy1 + hw / 4;
+			XFillPolygon(dpy, mapw->d, gc, points, 4,
+				     Convex, CoordModeOrigin);
+		    } else if (dir == right_dir(opposite_dir(viewdir))) {
+			points[0].x = sx;  points[0].y = sy + hw / 4;
+			points[1].x = sx + hw / 4;  points[1].y = sy + hw / 2;
+			points[2].x = sx + hw / 4;  points[2].y = sy1 + hw / 4;
+			points[3].x = sx;  points[3].y = sy1;
+			XFillPolygon(dpy, mapw->d, gc, points, 4,
+				     Convex, CoordModeOrigin);
+		    } else if (dir == viewdir) {
+			XDrawLine(dpy, mapw->d, gc,
+				  sx + hw / 4, sy, sx + hh - hw /4, sy);
+				  
+		    } else if (dir == left_dir(viewdir)) {
+			XDrawLine(dpy, mapw->d, gc,
+				  sx, sy + hw / 4, sx + hw /4, sy);
+				  
+		    } else if (dir == right_dir(viewdir)) {
+			XDrawLine(dpy, mapw->d, gc,
+				  sx + hh - hw / 4 - 1, sy, sx + hh - 1, sy + hw / 4);
+				  
+		    }
+		}
+	    }
+	}
+    }
+}
+
+int
+shading(int dir)
+{
+    switch (dir) {
+	case NORTHEAST:
+	  return 3;
+	case EAST:
+	  return 4;
+	case SOUTHEAST:
+	  return 3;
+	case SOUTHWEST:
+	  return 1;
+	case WEST:
+	  return 0;
+	case NORTHWEST:
+	  return 1;
+    }
+}
+
 /* Temporary stashes for the meridian drawing callbacks. */
 
 static MapW *tmpmapw;
@@ -900,7 +1139,6 @@ draw_meridians(MapW *mapw)
     Display *dpy = mapw->display;
 
     XSetClipMask(dpy, mapw->gc, None);
-    XSetLineAttributes(dpy, mapw->gc, 1, LineSolid, CapButt, JoinMiter); 
     XSetForeground(dpy, mapw->gc, dside->ui->meridian_color->pixel);
     XSetBackground(dpy, mapw->gc, dside->ui->bgcolor->pixel);
     tmpmapw = mapw;
@@ -1218,7 +1456,7 @@ draw_terrain_row(MapW *mapw, int x0, int y0, int len, int force)
 draw_terrain_transitions(MapW *mapw, int x0, int y0, int len, int force)
 {
     int x0w, x1, y1, x1w, x, xw, t, t1, t2, dir, sx, sy, sw, sh, sx2, sy2;
-    int sx3, sy3;
+    int sx3, sy3, quasirand;
     int x2, y2, trite, tleft, overrite, overleft, offset;
     int w = mapw->vp->hw, h = mapw->vp->hh, hch = mapw->vp->hch;
     int p = mapw->vp->power;
@@ -1235,6 +1473,9 @@ draw_terrain_transitions(MapW *mapw, int x0, int y0, int len, int force)
     for (x = x0; x < x0 + len + 1; ++x) {
 	xw = wrapx(x);
 	t = terrain_at(xw, y0);
+	drawmethod = cell_drawing_info(mapw, xw, y0, &pat, &color, &over);
+	if (drawmethod == dontdraw)
+	  continue;
 	for_all_directions(dir) {
 	    if (point_in_dir(xw, y0, dir, &x1, &y1)) {
 		t1 = terrain_at(x1, y1);
@@ -1251,10 +1492,14 @@ draw_terrain_transitions(MapW *mapw, int x0, int y0, int len, int force)
 		    overleft = (tleft == t1);
 		    drawmethod = cell_drawing_info(mapw, x1, y1,
 						   &pat, &color, &over);
+		    if (drawmethod == dontdraw) {
+			continue;
+		    }
 		    timg = best_image(dside->ui->timages[t1], w, h);
 		    xform(mapw, xw, y0, &sx, &sy);
 		    sx2 = sx - 2;  sy2 = sy - 2;
-		    sy2 -= ((xw + y0) % 4) * (h + 2);
+		    quasirand = (11 * xw + y0 + 7) * 41;
+		    sy2 -= (quasirand % 4) * (h + 2);
 		    switch (dir) {
 		      case NORTHEAST:
 			sx3 = sx + w / 2;  sy3 = sy;
@@ -1324,7 +1569,17 @@ draw_terrain_transitions(MapW *mapw, int x0, int y0, int len, int force)
 		    set_terrain_gc_for_image(mapw, gc, timg);
 		    XSetForeground(dpy, gc, color->pixel);
 		    XFillRectangle(dpy, mapw->d, gc, sx3, sy3, sw, sh);
-#if 0
+		    if (over < 0) {
+			enum grayshade shade = gray;
+
+			if (over == -2)
+			  shade = darkgray;
+			XSetFillStyle(dpy, gc, FillStippled);
+			XSetStipple(dpy, gc, dside->ui->grays[shade]);
+			XSetForeground(dpy, gc, dside->ui->blackcolor->pixel);
+			XFillRectangle(dpy, mapw->d, gc, sx3, sy3, sw, sh);
+		    }
+#if 0 /* for debugging */
 		    XSetClipMask(dpy, gc, None);
 		    XSetFillStyle(dpy, gc, FillSolid);
 		    XSetForeground(dpy, gc, dside->ui->badcolor->pixel);
@@ -1337,6 +1592,83 @@ draw_terrain_transitions(MapW *mapw, int x0, int y0, int len, int force)
     XSetFillStyle(dpy, gc, FillSolid);
 }
 
+/* Draw the fuzzed-out edges of the seen terrain all at once, using the
+   same general algorithm as for borders, but testing seen-ness. */
+
+static void
+draw_unseen_fuzz(MapW *mapw, int vx, int vy, int vw, int y1, int y2)
+{
+    int x, y, x1, len, bitmask, sx, sy, halfside;
+    int xw, yw, xne, yne, xnw, ynw;
+    int hw = mapw->vp->hw, hh = mapw->vp->hh, hch = mapw->vp->hch;
+    int power = mapw->vp->power;
+    Image *timg;
+    Display *dpy = mapw->display;
+    GC gc = mapw->gc;
+    XColor *color;
+	
+    if (power == 6)
+      halfside = 26;
+    else if (power == 5)
+      halfside = 13;
+    else
+      halfside = 7;
+    color = dside->ui->blackcolor;
+    /* (should use alternate unseen color if requested) */
+    XSetForeground(dpy, gc, color->pixel);
+    XSetClipMask(dpy, gc, fuzzpics[power]);
+    for (y = y1; y >= y2; --y) {
+	if (!compute_x1_len(mapw, vx, vy, vw, y, &x1, &len))
+	  continue;
+	for (x = x1; x < x1 + len; ++x) {
+	    /* Compute coordinates of adjacent cells. */
+	    if (!point_in_dir(x, y, NORTHEAST, &xne, &yne))
+	      continue;
+	    if (!point_in_dir(x, y, NORTHWEST, &xnw, &ynw))
+	      continue;
+	    if (!point_in_dir(x, y, WEST, &xw, &yw))
+	      continue;
+	    xform(mapw, x, y, &sx, &sy);
+	    /* Draw the junction at the top of the x,y hex. */
+	    bitmask = 0;
+	    if (terrain_visible(dside, xne, yne)
+		!= terrain_visible(dside, xnw, ynw))
+	      bitmask |= 1;
+	    if (terrain_visible(dside, x, y)
+		!= terrain_visible(dside, xne, yne))
+	      bitmask |= 2;
+	    if (terrain_visible(dside, x, y)
+		!= terrain_visible(dside, xnw, ynw))
+	      bitmask |= 4;
+	    if (bitmask != 0) {
+		XSetClipOrigin(dpy, gc,
+			       sx - (bitmask & 3) * hw,
+			       sy - halfside - ((bitmask & 4) ? 3 : 1) * hch);
+		XFillRectangle(dpy, mapw->d, gc, sx, sy - halfside, hw, hch);
+	    }
+	    /* Draw the junction at the top left corner of the x,y hex. */
+	    bitmask = 0;
+	    if (terrain_visible(dside, x, y)
+		!= terrain_visible(dside, xw, yw))
+	      bitmask |= 1;
+	    if (terrain_visible(dside, x, y)
+		!= terrain_visible(dside, xnw, ynw))
+	      bitmask |= 2;
+	    if (terrain_visible(dside, xw, yw)
+		!= terrain_visible(dside, xnw, ynw))
+	      bitmask |= 4;
+	    if (bitmask != 0) {
+		XSetClipOrigin(dpy, gc,
+			       sx - (bitmask & 3) * hw - hw / 2,
+			       sy - halfside - ((bitmask & 4) ? 2 : 0) * hch);
+		XFillRectangle(dpy, mapw->d, gc,
+			       sx - hw / 2, sy - halfside, hw, hch);
+	    }
+	}
+    }
+    XSetFillStyle(dpy, gc, FillSolid);
+}
+
 static void
 draw_contours(MapW *mapw, int x0, int y0, int len)
 {
@@ -1349,7 +1681,6 @@ draw_contours(MapW *mapw, int x0, int y0, int len)
     if (mapw->vp->contour_interval < 1)
       return;
     XSetClipMask(dpy, gc, None);
-    XSetLineAttributes(dpy, gc, 1, LineSolid, CapButt, JoinMiter); 
     XSetForeground(dpy, gc, dside->ui->contour_color->pixel);
     y = y0;
     for (xx = x0; xx < x0 + len; ++xx) {
@@ -1455,7 +1786,7 @@ draw_units(MapW *mapw, int x, int y)
 
     if ((mapw->map->see_all || units_visible(dside, xw, y))
 	&& unit_at(xw, y) != NULL) {
-	if (uw <= 16) {
+	if (uw <= 16 || mapw->vp->isometric) {
 	    /* At smaller mags, we choose a single best unit to display. */
 	    unit = NULL;
 	    for_all_stack(x, y, unit) {
@@ -1473,17 +1804,20 @@ draw_units(MapW *mapw, int x, int y)
 	    if (unit != NULL) {
 		xform(mapw, x, y, &sx, &sy);
 		/* Adjust to unit part of cell. */
-		sx += (mapw->vp->hw - uw) / 2;  sy += (mapw->vp->hh - uh) / 2;
+		if (mapw->vp->isometric) {
+		    sx += (mapw->vp->hh - uw) / 2; sy += (mapw->vp->hw / 2 - uh);
+		} else {
+		    sx += (mapw->vp->hw - uw) / 2;  sy += (mapw->vp->hh - uh) / 2;
+		}
 		if (unit->occupant != NULL
 		    && uw >= 8
+		    && !mapw->vp->isometric
 		    && (side_controls_unit(dside, unit)
 			|| mapw->map->see_all
 			|| u_see_occupants(unit->type))) {
 		    /* Draw a "grouping box", in white, but with no occs
 		       actually drawn. */
 		    XSetClipMask(dpy, gc, None);
-		    XSetLineAttributes(dpy, gc, 0, LineSolid, CapButt,
-				       JoinMiter);
 		    XSetForeground(dpy, gc, dside->ui->whitecolor->pixel);
 		    XFillRectangle(dpy, mapw->d, gc,
 				   sx + 1, sy + 1, uw - 2, uh - 2);
@@ -1565,7 +1899,6 @@ draw_unit_and_occs(MapW *mapw, Unit *unit, int sx, int sy, int sw, int sh,
 			 &sx2, &sy2, &sw2, &sh2);
 	/* Draw a white box to indicate the grouping. */
 	XSetClipMask(dpy, gc, None);
-	XSetLineAttributes(dpy, gc, 0, LineSolid, CapButt, JoinMiter);
 	XSetForeground(dpy, gc, dside->ui->whitecolor->pixel);
 	XFillRectangle(dpy, mapw->d, gc, sx + 1, sy + 1, sw - 2, sh - 2);
 	/* Put a black border around it, for better contrast. */
@@ -1589,9 +1922,12 @@ draw_unit_and_occs(MapW *mapw, Unit *unit, int sx, int sy, int sw, int sh,
 static void
 draw_unit_name(MapW *mapw, Unit *unit, int sx, int sy, int sw, int sh)
 {
-    if (!empty_string(unit->name)) {
+    char namebuf[BUFSIZE];
+
+    name_or_number(unit, namebuf);
+    if (!empty_string(namebuf)) {
 	draw_legend_text(mapw, sx + sw, sy + sh / 2 + 5,
-			 mapw->vp->power, unit->name,
+			 mapw->vp->power, namebuf,
 			 dside->ui->unit_name_color, TRUE);
     }
 }
@@ -1785,6 +2121,7 @@ draw_borders(MapW *mapw, int vx, int vy, int vw, int y1, int y2, int b)
 	}
     }
     XSetFillStyle(dpy, gc, FillSolid);
+    XSetLineAttributes(dpy, gc, 0, LineSolid, CapButt, JoinMiter); 
 }
 
 /* Draw all the connections of the given type that are visible. */
@@ -1810,7 +2147,11 @@ draw_connections(MapW *mapw, int vx, int vy, int vw, int y1, int y2, int c)
     timg = best_image(dside->ui->timages[c], wid, wid);
     set_terrain_gc_for_image(mapw, gc, timg);
     if (!mapw->draw_lines && (power == 4 || power == 5)) {
-	XSetClipMask(dpy, gc, connpics[power]);
+	if (strcmp(t_type_name(c), "railroad") == 0
+	    || strcmp(t_type_name(c), "railway") == 0)
+	  XSetClipMask(dpy, gc, rrpics[power]);
+	else
+	  XSetClipMask(dpy, gc, connpics[power]);
     } else {
 	XSetClipMask(dpy, gc, None);
     }
@@ -1848,6 +2189,7 @@ draw_connections(MapW *mapw, int vx, int vy, int vw, int y1, int y2, int c)
 	}
     }
     XSetFillStyle(dpy, gc, FillSolid);
+    XSetLineAttributes(dpy, gc, 0, LineSolid, CapButt, JoinMiter); 
 }
 
 /* Draw any text that should be associated with this cell. */
@@ -1920,17 +2262,18 @@ draw_current(MapW *mapw)
 	sw = uw;  sh = uh;
     }
     /* Always redraw the unit that the cursor is showing. */
-    draw_unit_image(mapw, sx, sy, sw, sh, unit->type, -1, !completed(unit));
+    draw_unit_image(mapw, sx, sy, sw, sh, unit->type, side_number(unit->side),
+		    !completed(unit));
     if (mapw->vp->draw_names)
       draw_unit_name(mapw, unit, sx, sy, sw, sh);
-    /* "Outdent" the curunit indicator slightly. */
+    /* "Outdent" the curunit indicator slightly, so that it doesn't cover
+       up some of the unit image. */
     if (sw >= 16) {
 	sx -= 1;  sy -= 1;
 	sw += 2;  sh += 2;
     }
     /* Draw the curunit indicator proper. */
     XSetClipMask(dpy, gc, None);
-    XSetLineAttributes(dpy, gc, 0, LineSolid, CapButt, JoinMiter); 
     if (mapw->map->autoselect) {
 	XSetBackground(dpy, gc, dside->ui->blackcolor->pixel);
 	XSetFillStyle(dpy, gc, FillOpaqueStippled);
@@ -1981,7 +2324,12 @@ draw_blast(Map *map, Unit *unit, int blasttype)
     MapW *mapw = (MapW *) map->widget;
 
     if (between(0, blasttype, 2)) {
-	x_xform_unit(mapw, unit, &sx, &sy, &sw, &sh);
+	if (mapw->vp->hw > 10) {
+	    x_xform_unit(mapw, unit, &sx, &sy, &sw, &sh);
+	} else {
+	    xform(mapw, unit->x, unit->y, &sx, &sy);
+	    sw = mapw->vp->hw;  sh = mapw->vp->hh;
+	}
 	mapw->blastsx = sx;  mapw->blastsy = sy;
 	mapw->blastsw = sw;  mapw->blastsh = sh;
 	mapw->blasttype = blasttype;
@@ -2012,7 +2360,8 @@ draw_blast_image(MapW *mapw, int sx, int sy, int sw, int sh, int blasttype)
 
     XSetClipMask(dpy, gc, None);
     if (blasttype == 3) {
-	XSetFunction(dpy, gc, GXinvert);
+	/*	XSetFunction(dpy, gc, GXinvert); */
+	XSetForeground(dpy, gc, dside->ui->badcolor->pixel);
 	XFillRectangle(dpy, mapw->d, gc, sx, sy, sw, sh);
 	XSetFunction(dpy, gc, GXcopy);
     } else if (sw >= 16 && sh >= 16) {
@@ -2063,7 +2412,7 @@ draw_blast_image(MapW *mapw, int sx, int sy, int sw, int sh, int blasttype)
 	XSetForeground(dpy, gc, dside->ui->badcolor->pixel);
 	XFillRectangle(dpy, mapw->d, gc, sx, sy, sw - 1, sh - 1);
     } else {
-	/* Draw a red box only. */
+	/* At the smallest sizes, draw a red box only. */
 	XSetForeground(dpy, gc, dside->ui->badcolor->pixel);
 	XFillRectangle(dpy, mapw->d, gc, sx, sy, sw, sh);
     }
@@ -2115,6 +2464,7 @@ draw_feature_boundary(MapW *mapw, int x, int y, int fid)
 	    }
 	}
     }
+    XSetLineAttributes(dpy, gc, 0, LineSolid, CapButt, JoinMiter); 
 }
 
 /* Do the grody work of drawing very large polygons accurately. */
@@ -2209,7 +2559,6 @@ int sx, sy, sw, sh, u, s2, mod;
     if (sw <= 0)
       return;
     XSetClipMask(dpy, gc, None);
-    XSetLineAttributes(dpy, gc, 0, LineSolid, CapButt, JoinMiter);
     /* Just draw a small box at 4x4 and below. */
     if (sw <= 4) {
 	/* (should draw in side color if possible) */
@@ -2416,6 +2765,7 @@ int sx, sy, dir, con, heavy;
 	      sx + bsx[pow][dir], sy + bsy[pow][dir],
 	      sx + bsx[pow][dir+1], sy + bsy[pow][dir+1]);
     XSetFillStyle(dpy, gc, FillSolid);
+    XSetLineAttributes(dpy, gc, 0, LineSolid, CapButt, JoinMiter);
 }
 
 /* Put the point x, y in the center of the mapw, or at least as close
@@ -2611,7 +2961,9 @@ handle_world_mouse_up(Map *map, int sx, int sy, int button)
 VP *
 widget_vp(Map *map)
 {
-    return ((MapW *) map->widget)->vp;
+    MapW *mapw = (MapW *) map->widget;
+
+    return (mapw ? mapw->vp : NULL);
 }
 
 /* Force a refresh of the map widgets in a map. */
@@ -2673,6 +3025,15 @@ handle_designer_mouse_down(Side *side, Map *map, int sx, int sy)
 	painttype = (dside->ui->curttype == oldt ? dside->ui->curbgttype : dside->ui->curttype);
 	net_paint_cell(dside, x, y, dside->ui->curbrushradius, painttype);
 	break;
+      case bord_paint_mode:
+	net_paint_border(dside, x, y, dir, dside->ui->curttype, -1);
+	break;
+      case conn_paint_mode:
+	net_paint_connection(dside, x, y, dir, dside->ui->curttype, -1);
+	break;
+      case coat_paint_mode:
+	/* (should do something) */
+	break;
       case unit_paint_mode:
 	unit = net_designer_create_unit(dside, dside->ui->curutype,
 					dside->ui->curusidenumber, x, y);
@@ -2782,8 +3143,8 @@ limitx(int x, int y)
 	x = limitn(0, x, area.width - 1);
 	if (x + y < area.halfheight)
 	  x = area.halfheight - y;
-	if (x + y > area.width + area.halfheight)
-	  x = area.width - (area.halfheight + y);
+	if (x + y >= area.width + area.halfheight)
+	  x = area.width - (area.halfheight + y) - 1;
     }
     return x;
 }
@@ -2798,15 +3159,13 @@ update_at_cell(Map *map, int x, int y, int flags)
     mapw = (MapW *) map->widget;
     if (mapw == NULL)
       return;
-#if 0
-    if (!((flags & UPDATE_RIGHTNOW)
+    if (!((flags & UPDATE_ALWAYS)
 	  || ((flags & UPDATE_COVER) && mapw->vp->draw_cover)
 	  || ((flags & UPDATE_TEMP) && mapw->vp->draw_temperature)
 	  || ((flags & UPDATE_CLOUDS) && mapw->vp->draw_clouds)
 	  || ((flags & UPDATE_WINDS) && mapw->vp->draw_winds)
 	  ))
       return;
-#endif
     if (mapw->rsw >= 0) {
 	printf("rsw is already %d\n", mapw->rsw);
 	eval_tcl_cmd("update idletasks");
@@ -2832,6 +3191,8 @@ update_at_unit(Map *map, Unit *unit)
     if (!in_area(unit->x, unit->y))
       return;
     mapw = (MapW *) map->widget;
+    if (mapw == NULL)
+      return;
     if (mapw->rsw >= 0) {
 	printf("rsw is already %d\n", mapw->rsw);
 	eval_tcl_cmd("update idletasks");
@@ -2842,20 +3203,24 @@ update_at_unit(Map *map, Unit *unit)
     xform(mapw, unit->x, unit->y, &sx, &sy);
     mapw->rsx = sx;  mapw->rsy = sy;
     mapw->rsw = mapw->vp->hw;  mapw->rsh = mapw->vp->hh;
-#if 0
-    extraw = mapw->vp->hw / 2;  extrah = mapw->vp->hh / 2;
+    extraw = extrah = 0;
+    if (0 /*mapw->draw_transitions || !g_terrain_seen()*/) {
+	extraw = mapw->vp->hw;  extrah = mapw->vp->hh;
+    }
     mapw->rsx -= extraw;  mapw->rsy -= extrah;
     mapw->rsw += 2 * extraw;  mapw->rsh += 2 * extrah;
-#endif
     mapw = (MapW *) map->worldw;
+    if (mapw == NULL)
+      return;
     xform(mapw, unit->x, unit->y, &sx, &sy);
     mapw->rsx = sx;  mapw->rsy = sy;
     mapw->rsw = mapw->vp->hw;  mapw->rsh = mapw->vp->hh;
-#if 0
-    extraw = mapw->vp->hw / 2;  extrah = mapw->vp->hh / 2;
+    extraw = extrah = 0;
+    if (0 /*mapw->draw_transitions || !g_terrain_seen()*/) {
+	extraw = mapw->vp->hw;  extrah = mapw->vp->hh;
+    }
     mapw->rsx -= extraw;  mapw->rsy -= extrah;
     mapw->rsw += 2 * extraw;  mapw->rsh += 2 * extrah;
-#endif
     redraw_map(map);
 }
 
@@ -2888,6 +3253,16 @@ toggle_mapw_transitions(Map *map)
       mapw->draw_transitions = !mapw->draw_transitions;
 }
 
+mapw_transitions(Map *map)
+{
+    MapW *mapw;
+
+    mapw = (MapW *) map->widget;
+    if (mapw)
+      return mapw->draw_transitions;
+    return FALSE;
+}
+
 draw_fire_line(Map *map, Unit *unit, Unit *unit2, int x2, int y2)
 {
     int i, sx1, sy1, sw1, sh1, sx2, sy2, sw2, sh2, dx, dy, xx, yy;
@@ -2902,7 +3277,6 @@ draw_fire_line(Map *map, Unit *unit, Unit *unit2, int x2, int y2)
 	GC gc = mapw->gc;
 
 	XSetClipMask(dpy, gc, None);
-	XSetLineAttributes(dpy, gc, 1, LineSolid, CapButt, JoinMiter);
 	XSetFunction(dpy, gc, GXinvert);
 	for (i = 0; i < 32; ++i) {
 	    x_xform_unit_self(mapw, unit, &sx1, &sy1, &sw1, &sh1);
